@@ -27,19 +27,36 @@ import numpy as np
 from .core import LRCP, RLCP, RPCL, PCRL, CPRL
 from .core import WAVELET_XFORM_9X7_IRREVERSIBLE
 from .core import WAVELET_XFORM_5X3_REVERSIBLE
-from .core import _CAPABILITIES_DISPLAY
+from .core import _Keydefaultdict
 from .lib import openjp2 as opj2
 
-_PROGRESSION_ORDER_DISPLAY = {
-    LRCP: 'LRCP',
-    RLCP: 'RLCP',
-    RPCL: 'RPCL',
-    PCRL: 'PCRL',
-    CPRL: 'CPRL'}
+_factory = lambda x:  '{0} (invalid)'.format(x)
+_PROGRESSION_ORDER_DISPLAY = _Keydefaultdict(_factory,
+        { LRCP: 'LRCP',
+          RLCP: 'RLCP',
+          RPCL: 'RPCL',
+          PCRL: 'PCRL',
+          CPRL: 'CPRL'})
 
-_WAVELET_TRANSFORM_DISPLAY = {
-    WAVELET_XFORM_9X7_IRREVERSIBLE: '9-7 irreversible',
-    WAVELET_XFORM_5X3_REVERSIBLE: '5-3 reversible'}
+_WAVELET_TRANSFORM_DISPLAY = _Keydefaultdict(_factory,
+        { WAVELET_XFORM_9X7_IRREVERSIBLE: '9-7 irreversible',
+          WAVELET_XFORM_5X3_REVERSIBLE: '5-3 reversible'})
+
+_NO_PROFILE = 0
+_PROFILE_0 = 1
+_PROFILE_1 = 2
+_PROFILE_3 = 3
+_PROFILE_4 = 4
+
+_KNOWN_PROFILES = [_NO_PROFILE, _PROFILE_0, _PROFILE_1, _PROFILE_3, _PROFILE_4]
+
+# How to display the codestream profile.
+_CAPABILITIES_DISPLAY = _Keydefaultdict(_factory,
+        { _NO_PROFILE: 'no profile',
+          _PROFILE_0: '0',
+          _PROFILE_1: '1',
+          _PROFILE_3: 'Cinema 2K',
+          _PROFILE_4: 'Cinema 4K'} )
 
 # Need a catch-all list of valid markers.
 # See table A-1 in ISO/IEC FCD15444-1.
@@ -171,15 +188,7 @@ class Codestream(object):
         while True:
 
             read_buffer = fptr.read(2)
-            try:
-                self._marker_id, = struct.unpack('>H', read_buffer)
-            except struct.error:
-                # Treat this as a warning.
-                msg = "Marker had length {0} instead of expected length of 2 "
-                msg += "bytes.  Codestream parsing terminated."
-                warnings.warn(msg.format(len(read_buffer)))
-                break
-
+            self._marker_id, = struct.unpack('>H', read_buffer)
             self._offset = fptr.tell() - 2
 
             if self._marker_id == 0xff90 and header_only:
@@ -363,14 +372,22 @@ class Codestream(object):
         COD segment instance.
         """
         offset = fptr.tell() - 2
-        offset = fptr.tell() - 2
 
-        read_buffer = fptr.read(3)
-        length, scod = struct.unpack('>HB', read_buffer)
+        read_buffer = fptr.read(2)
+        length, = struct.unpack('>H', read_buffer)
 
-        numbytes = offset + 2 + length - fptr.tell()
-        spcod = fptr.read(numbytes)
+        read_buffer = fptr.read(length - 2)
+        scod, = struct.unpack_from('>B', read_buffer, offset=0)
+        spcod = read_buffer[1:]
         spcod = np.frombuffer(spcod, dtype=np.uint8)
+        if spcod[0] not in [LRCP, RLCP, RPCL, PCRL, CPRL]:
+            msg = "Invalid progression order in COD segment: {0}."
+            warnings.warn(msg.format(spcod[0]))
+
+        if spcod[8] not in [WAVELET_XFORM_9X7_IRREVERSIBLE,
+                            WAVELET_XFORM_5X3_REVERSIBLE]:
+            msg = "Invalid wavelet transform in COD segment: {0}."
+            warnings.warn(msg.format(spcod[8]))
 
         sop = (scod & 2) > 0
         eph = (scod & 4) > 0
@@ -558,22 +575,21 @@ class Codestream(object):
         read_buffer = fptr.read(2)
         length, = struct.unpack('>H', read_buffer)
 
+        read_buffer = fptr.read(length - 2)
         if self._csiz > 256:
-            read_buffer = fptr.read(3)
             fmt = '>HB'
-            mantissa_exponent_buffer_length = length - 5
+            mantissa_exponent_offset = 3
         else:
-            read_buffer = fptr.read(2)
             fmt = '>BB'
-            mantissa_exponent_buffer_length = length - 4
-        cqcc, sqcc = struct.unpack(fmt, read_buffer)
+            mantissa_exponent_offset = 2
+        cqcc, sqcc = struct.unpack_from(fmt, read_buffer)
         if cqcc >= self._csiz:
             msg = "Invalid component number ({0}), "
             msg += "number of components is only {1}."
             msg = msg.format(cqcc, self._csiz)
             warnings.warn(msg)
 
-        spqcc = fptr.read(mantissa_exponent_buffer_length)
+        spqcc = read_buffer[mantissa_exponent_offset:]
 
         return QCCsegment(cqcc, sqcc, spqcc, length, offset)
 
@@ -645,17 +661,64 @@ class Codestream(object):
         read_buffer = fptr.read(2)
         length, = struct.unpack('>H', read_buffer)
 
-        xy_buffer = fptr.read(36)
+        read_buffer = fptr.read(length - 2)
+        data = struct.unpack_from('>HIIIIIIIIH', read_buffer)
 
-        num_components, = struct.unpack('>H', xy_buffer[-2:])
+        rsiz = data[0]
+        if rsiz not in _KNOWN_PROFILES:
+            warnings.warn("Invalid profile: (Rsiz={0}).".format(rsiz))
 
-        component_buffer = fptr.read(num_components * 3)
+        xysiz = (data[1], data[2])
+        xyosiz = (data[3], data[4])
+        xytsiz = (data[5], data[6])
+        xytosiz = (data[7], data[8])
 
-        segment = SIZsegment(xy_buffer, component_buffer, length, offset)
+        # Csiz is the number of components
+        Csiz = data[9]
+
+        data = struct.unpack_from('>' + 'B' * (length - 36 - 2),
+                                  read_buffer, offset=36)
+
+        bitdepth = tuple(((x & 0x7f) + 1) for x in data[0::3])
+        signed = tuple(((x & 0x80) > 0) for x in data[0::3])
+        xrsiz = data[1::3]
+        yrsiz = data[2::3]
+
+        for j, subsampling in enumerate(zip(xrsiz, yrsiz)):
+            if 0 in subsampling:
+                msg = "Invalid subsampling value for component {0}: "
+                msg += "dx={1}, dy={2}."
+                msg = msg.format(j, subsampling[0], subsampling[1])
+                warnings.warn(msg)
+
+        try:
+            num_tiles_x = (xysiz[0] - xyosiz[0]) / (xytsiz[0] - xytosiz[0])
+            num_tiles_y = (xysiz[1] - xyosiz[1]) / (xytsiz[1] - xytosiz[1])
+        except ZeroDivisionError as err:
+            warnings.warn("Invalid tile dimensions.")
+        else:
+            numtiles = math.ceil(num_tiles_x) * math.ceil(num_tiles_y)
+            if numtiles > 65535:
+                msg = "Invalid number of tiles ({0}).".format(numtiles)
+                warnings.warn(msg)
+
+
+        kwargs = {'rsiz': rsiz,
+                  'xysiz': xysiz,
+                  'xyosiz': xyosiz,
+                  'xytsiz': xytsiz,
+                  'xytosiz': xytosiz,
+                  'Csiz': Csiz,
+                  'bitdepth': bitdepth,
+                  'signed':  signed,
+                  'xyrsiz': (xrsiz, yrsiz),
+                  'length': length,
+                  'offset': offset}
+        segment = SIZsegment(**kwargs)
 
         # Need to keep track of the number of components from SIZ for
-        # other markers
-        self._csiz = len(segment.ssiz)
+        # other markers.
+        self._csiz = Csiz
 
         return segment
 
@@ -731,8 +794,8 @@ class Codestream(object):
         read_buffer = fptr.read(2)
         length, = struct.unpack('>H', read_buffer)
 
-        read_buffer = fptr.read(2)
-        ztlm, stlm = struct.unpack('>BB', read_buffer)
+        read_buffer = fptr.read(length - 2)
+        ztlm, stlm = struct.unpack_from('>BB', read_buffer)
         ttlm_st = (stlm >> 4) & 0x3
         ptlm_sp = (stlm >> 6) & 0x1
 
@@ -742,7 +805,6 @@ class Codestream(object):
         else:
             ntiles = nbytes / (ttlm_st + (ptlm_sp + 1) * 2)
 
-        read_buffer = fptr.read(nbytes)
         if ttlm_st == 0:
             ttlm = None
             fmt = ''
@@ -756,7 +818,8 @@ class Codestream(object):
         else:
             fmt += 'I'
 
-        data = struct.unpack('>' + fmt * int(ntiles), read_buffer)
+        data = struct.unpack_from('>' + fmt * int(ntiles), read_buffer,
+                                  offset=2)
         if ttlm_st == 0:
             ttlm = None
             ptlm = data
@@ -1441,8 +1504,8 @@ class SIZsegment(Segment):
         Width and height of reference tile with respect to the reference grid.
     xtosiz, ytosiz : int
         Horizontal and vertical offsets of tile from origin of reference grid.
-    ssiz : iterable bytes
-        Encoded precision (depth) in bits and sign of each component.
+    Csiz : int
+        Number of components in image.
     bitdepth : iterable bytes
         Precision (depth) in bits of each component.
     signed : iterable bool
@@ -1457,48 +1520,45 @@ class SIZsegment(Segment):
        15444-1:2004 - Information technology -- JPEG 2000 image coding system:
        Core coding system
     """
-    def __init__(self, xy_buffer, component_buffer, length, offset):
-        Segment.__init__(self, marker_id='SIZ')
+    def __init__(self, rsiz=-1, xysiz=None, xyosiz=-1, xytsiz=-1, xytosiz=-1,
+                 Csiz=-1, bitdepth=None, signed=None, xyrsiz=-1, length=-1,
+                 offset=-1):
+        Segment.__init__(self, marker_id='SIZ', length=length, offset=offset)
 
-        data = struct.unpack('>HIIIIIIIIH', xy_buffer)
+        self.rsiz = rsiz
+        self.xsiz, self.ysiz = xysiz
+        self.xosiz, self.yosiz = xyosiz
+        self.xtsiz, self.ytsiz = xytsiz
+        self.xtosiz, self.ytosiz = xytosiz
+        self.Csiz = Csiz
+        self.bitdepth = bitdepth
+        self.signed = signed
+        self.xrsiz, self.yrsiz = xyrsiz
 
-        self.rsiz = data[0]
-        self.xsiz = data[1]
-        self.ysiz = data[2]
-        self.xosiz = data[3]
-        self.yosiz = data[4]
-        self.xtsiz = data[5]
-        self.ytsiz = data[6]
-        self.xtosiz = data[7]
-        self.ytosiz = data[8]
-        # disregarding the last element in data
+        # ssiz attribute to be removed in 1.0.0
+        lst = []
+        for bitdepth, signed in zip(self.bitdepth, self.signed):
+            if signed:
+                lst.append((bitdepth - 1) | 0x80)
+            else:
+                lst.append(bitdepth - 1)
+        self.ssiz = tuple(lst)
 
-        num_tiles_x = (self.xsiz - self.xosiz) / (self.xtsiz - self.xtosiz)
-        num_tiles_y = (self.ysiz - self.yosiz) / (self.ytsiz - self.ytosiz)
-        numtiles = math.ceil(num_tiles_x) * math.ceil(num_tiles_y)
-        if numtiles > 65535:
-            msg = "Invalid number of tiles ({0}).".format(numtiles)
-            warnings.warn(msg)
-
-        data = struct.unpack('>' + 'B' * len(component_buffer),
-                             component_buffer)
-
-        self.ssiz = data[0::3]
-
-        for j, subsampling in enumerate(list(zip(data[1::3], data[2::3]))):
-            if 0 in subsampling:
-                msg = "Invalid subsampling value for component {0}: "
-                msg += "dx={1}, dy={2}."
-                msg = msg.format(j, subsampling[0], subsampling[1])
-                warnings.warn(msg)
-        self.xrsiz = data[1::3]
-        self.yrsiz = data[2::3]
-
-        self.bitdepth = tuple(((x & 0x7f) + 1) for x in self.ssiz)
-        self.signed = tuple(((x & 0xb0) > 0) for x in self.ssiz)
-
-        self.length = length
-        self.offset = offset
+    def __repr__(self):
+        msg = "glymur.codestream.SIZsegment(rsiz={rsiz}, xysiz={xysiz}, "
+        msg += "xyosiz={xyosiz}, xytsiz={xytsiz}, xytosiz={xytosiz}, "
+        msg += "Csiz={Csiz}, bitdepth={bitdepth}, signed={signed}, "
+        msg += "xyrsiz={xyrsiz})"
+        msg = msg.format(rsiz=self.rsiz,
+                         xysiz=(self.xsiz, self.ysiz),
+                         xyosiz=(self.xosiz, self.yosiz),
+                         xytsiz=(self.xtsiz, self.ytsiz),
+                         xytosiz=(self.xtosiz, self.ytosiz),
+                         Csiz=self.Csiz,
+                         bitdepth=self.bitdepth,
+                         signed=self.signed,
+                         xyrsiz=(self.xrsiz, self.yrsiz))
+        return msg
 
     def __str__(self):
         msg = Segment.__str__(self)
@@ -1549,6 +1609,9 @@ class SOCsegment(Segment):
         Segment.__init__(self, marker_id='SOC')
         self.__dict__.update(**kwargs)
 
+    def __repr__(self):
+        msg = "glymur.codestream.SOCsegment()"
+        return msg
 
 class SODsegment(Segment):
     """Container for Start of Data (SOD) segment information.
