@@ -16,11 +16,13 @@ if sys.hexversion >= 0x03030000:
 else:
     from contextlib2 import ExitStack
 
+from collections import Counter
 import ctypes
 import math
 import os
 import re
 import struct
+from uuid import UUID
 import warnings
 
 import numpy as np
@@ -29,6 +31,7 @@ from .codestream import Codestream
 from .core import SRGB, GREYSCALE
 from .core import PROGRESSION_ORDER
 from .core import ENUMERATED_COLORSPACE, RESTRICTED_ICC_PROFILE
+from . import core
 from .jp2box import Jp2kBox
 from .jp2box import JPEG2000SignatureBox, FileTypeBox, JP2HeaderBox
 from .jp2box import ColourSpecificationBox, ContiguousCodestreamBox
@@ -38,6 +41,10 @@ from .lib import openjp2 as opj2
 from . import version
 from .lib import c as libc
 
+JP2_IDS = ['colr', 'cdef', 'cmap', 'jp2c', 'ftyp', 'ihdr', 'jp2h', 'jP  ',
+           'pclr', 'res ', 'resc', 'resd', 'xml ', 'ulst', 'uinf', 'url ',
+           'uuid']
+JPX_IDS = ['asoc', 'nlst']
 
 class Jp2k(Jp2kBox):
     """JPEG 2000 file.
@@ -73,6 +80,10 @@ class Jp2k(Jp2kBox):
         if mode == 'rb':
             self.parse()
 
+    def __repr__(self):
+        msg = "glymur.Jp2k('{0}')".format(self.filename)
+        return msg
+
     def __str__(self):
         metadata = ['File:  ' + os.path.basename(self.filename)]
         if len(self.box) > 0:
@@ -144,6 +155,56 @@ class Jp2k(Jp2kBox):
                     msg += "profile if the file type box brand is 'jp2 '."
                     warnings.warn(msg)
 
+    def _set_cinema_params(self, cparams, cinema_mode, fps):
+        """Populate compression parameters structure for cinema2K.
+
+        Parameters
+        ----------
+        params : ctypes struct
+            Corresponds to compression parameters structure used by the
+            library.
+        cinema_mode : str
+            Either 'cinema2k' or 'cinema4k'
+        fps : int
+            Frames per second, should be either 24 or 48.
+        """
+        if re.match("(1.5|2.0.0)", version.openjpeg_version) is not None:
+            msg = "Writing Cinema2K or Cinema4K files is not supported with "
+            msg += 'openjpeg library versions less than 2.0.1.'
+            raise IOError(msg)
+
+        if cinema_mode == 'cinema2k':
+            if fps not in [24, 48]:
+                raise IOError('Cinema2K frame rate must be either 24 or 48.')
+
+            if re.match("2.0", version.openjpeg_version) is not None:
+                # 2.0 API
+                if fps == 24:
+                    cparams.cp_cinema = core.OPJ_CINEMA2K_24
+                else:
+                    cparams.cp_cinema = core.OPJ_CINEMA2K_48
+            else:
+                # 2.1 API
+                if fps == 24:
+                    cparams.rsiz = core.OPJ_PROFILE_CINEMA_2K
+                    cparams.max_comp_size = core.OPJ_CINEMA_24_COMP
+                    cparams.max_cs_size = core.OPJ_CINEMA_24_CS
+                else:
+                    cparams.rsiz = core.OPJ_PROFILE_CINEMA_2K
+                    cparams.max_comp_size = core.OPJ_CINEMA_48_COMP
+                    cparams.max_cs_size = core.OPJ_CINEMA_48_CS
+
+        else:
+            # cinema4k
+            if re.match("2.0", version.openjpeg_version) is not None:
+                # 2.0 API
+                cparams.cp_cinema = core.OPJ_CINEMA4K_24
+            else:
+                # 2.1 API
+                cparams.rsiz = core.OPJ_PROFILE_CINEMA_4K
+
+        return
+
     def _populate_cparams(self, **kwargs):
         """Populate compression parameters structure from input arguments.
 
@@ -210,6 +271,17 @@ class Jp2k(Jp2kBox):
         cparams.tcp_numlayers = 1
         cparams.cp_disto_alloc = 1
 
+        if 'irreversible' in kwargs and kwargs['irreversible'] is True:
+            cparams.irreversible = 1
+
+        if 'cinema2k' in kwargs:
+            self._set_cinema_params(cparams, 'cinema2k', kwargs['cinema2k'])
+            return cparams
+
+        if 'cinema4k' in kwargs:
+            self._set_cinema_params(cparams, 'cinema4k', kwargs['cinema4k'])
+            return cparams
+
         if 'cbsize' in kwargs:
             cparams.cblockw_init = kwargs['cbsize'][1]
             cparams.cblockh_init = kwargs['cbsize'][0]
@@ -289,6 +361,10 @@ class Jp2k(Jp2kBox):
         colorspace : int
             Either CLRSPC_SRGB or CLRSPC_GRAY
         """
+        if (('cinema2k' in kwargs or 'cinema4k' in kwargs)  and
+                (len(set(kwargs)) > 1)):
+            msg = "Cannot specify cinema2k/cinema4k along with other options."
+            raise IOError(msg)
 
         if 'cratios' in kwargs and 'psnr' in kwargs:
             msg = "Cannot specify cratios and psnr together."
@@ -331,6 +407,10 @@ class Jp2k(Jp2kBox):
             Image data to be written to file.
         cbsize : tuple, optional
             Code block size (DY, DX).
+        cinema2k : int, optional
+            frames per second, either 24 or 48
+        cinema4k : bool, optional
+            Set to True to specify Cinema4K mode, defaults to false.
         colorspace : str, optional
             Either 'rgb' or 'gray'.
         cratios : iterable
@@ -339,6 +419,8 @@ class Jp2k(Jp2kBox):
             If true, write SOP marker after each header packet.
         grid_offset : tuple, optional
             Offset (DY, DX) of the origin of the image in the reference grid.
+        irreversible : bool, optional
+            If true, use the irreversible DWT 9-7 transform. 
         mct : bool, optional
             Specifies usage of the multi component transform.  If not
             specified, defaults to True if the colorspace is RGB.
@@ -386,7 +468,7 @@ class Jp2k(Jp2kBox):
             If glymur is unable to load the openjp2 library.
         """
         if opj2.OPENJP2 is not None:
-            self._write_openjp2(img_array,  verbose=verbose, **kwargs)
+            self._write_openjp2(img_array, verbose=verbose, **kwargs)
         elif opj.OPENJPEG is not None:
             self._write_openjpeg(img_array, verbose=verbose, **kwargs)
         else:
@@ -464,7 +546,7 @@ class Jp2k(Jp2kBox):
 
     def _write_openjp2(self, img_array, verbose=False, **kwargs):
         """
-        Write JPEG 2000 file using OpenJPEG 1.5 interface.
+        Write JPEG 2000 file using OpenJPEG 2.0 interface.
         """
         cparams, colorspace = self._process_write_inputs(img_array, **kwargs)
 
@@ -480,32 +562,32 @@ class Jp2k(Jp2kBox):
             stack.callback(opj2.image_destroy, image)
 
             _populate_image_struct(cparams, image, img_array)
-    
+
             codec = opj2.create_compress(cparams.codec_fmt)
             stack.callback(opj2.destroy_codec, codec)
-    
+
             info_handler = _INFO_CALLBACK if verbose else None
             opj2.set_info_handler(codec, info_handler)
             opj2.set_warning_handler(codec, _WARNING_CALLBACK)
             opj2.set_error_handler(codec, _ERROR_CALLBACK)
-    
+
             opj2.setup_encoder(codec, cparams, image)
-    
-            if re.match("2.0", version.openjpeg_version):
+
+            if re.match("2.0", version.openjpeg_version) is not None:
                 fptr = libc.fopen(self.filename, 'wb')
                 strm = opj2.stream_create_default_file_stream(fptr, False)
                 stack.callback(opj2.stream_destroy, strm)
                 stack.callback(libc.fclose, fptr)
             else:
-                # This routine introduced in 2.1.
+                # Introduced in 2.1 devel series.
                 strm = opj2.stream_create_default_file_stream(self.filename,
-                                                                 False)
+                                                              False)
                 stack.callback(opj2.stream_destroy, strm)
-    
+
             opj2.start_compress(codec, image, strm)
             opj2.encode(codec, strm)
             opj2.end_compress(codec, strm)
-    
+
         # Refresh the metadata.
         self.parse()
 
@@ -515,14 +597,18 @@ class Jp2k(Jp2kBox):
         Parameters
         ----------
         box : Jp2Box
-            Instance of a JP2 box.  Currently only XML boxes are allowed.
+            Instance of a JP2 box.  Only UUID and XML boxes can currently be
+            appended.
         """
         if self._codec_format == opj2.CODEC_J2K:
             msg = "Only JP2 files can currently have boxes appended to them."
             raise IOError(msg)
 
-        if box.box_id != 'xml ':
-            raise IOError("Only XML boxes can currently be appended.")
+        if not ((box.box_id == 'xml ') or
+                (box.box_id == 'uuid' and
+                 box.uuid == UUID('be7acfcb-97a9-42e8-9c71-999491e3afac'))):
+            msg = "Only XML boxes and XMP UUID boxes can currently be appended."
+            raise IOError(msg)
 
         # Check the last box.  If the length field is zero, then rewrite
         # the length field to reflect the true length of the box.
@@ -546,7 +632,12 @@ class Jp2k(Jp2kBox):
         self.parse()
 
     def wrap(self, filename, boxes=None):
-        """Write the codestream back out to file, wrapped in new JP2 jacket.
+        """Create a new JP2/JPX file wrapped in a new set of JP2 boxes.
+
+        This method is primarily aimed at wrapping a raw codestream in a set of
+        of JP2 boxes (turning it into a JP2 file instead of just a raw
+        codestream), or rewrapping a codestream in a JP2 file in a new "jacket"
+        of JP2 boxes.
 
         Parameters
         ----------
@@ -556,6 +647,8 @@ class Jp2k(Jp2kBox):
             JP2 box definitions to define the JP2 file format.  If not
             provided, a default ""jacket" is assumed, consisting of JP2
             signature, file type, JP2 header, and contiguous codestream boxes.
+            A JPX file rewrapped without the boxes argument results in a JP2
+            file encompassing the first codestream.
 
         Returns
         -------
@@ -571,19 +664,7 @@ class Jp2k(Jp2kBox):
         >>> jp2 = j2k.wrap(tfile.name)
         """
         if boxes is None:
-            # Try to create a reasonable default.
-            boxes = [JPEG2000SignatureBox(),
-                     FileTypeBox(),
-                     JP2HeaderBox(),
-                     ContiguousCodestreamBox()]
-            codestream = self.get_codestream()
-            height = codestream.segment[1].ysiz
-            width = codestream.segment[1].xsiz
-            num_components = len(codestream.segment[1].xrsiz)
-            boxes[2].box = [ImageHeaderBox(height=height,
-                                           width=width,
-                                           num_components=num_components),
-                            ColourSpecificationBox(colorspace=SRGB)]
+            boxes = self._get_default_jp2_boxes()
 
         _validate_jp2_box_sequence(boxes)
 
@@ -592,34 +673,92 @@ class Jp2k(Jp2kBox):
                 if box.box_id != 'jp2c':
                     box.write(ofile)
                 else:
-                    # The codestream gets written last.
-                    if len(self.box) == 0:
-                        # Am I a raw codestream?  If so, then it is pretty
-                        # easy, just write the codestream box header plus all
-                        # of myself out to file.
-                        ofile.write(struct.pack('>I', self.length + 8))
-                        ofile.write('jp2c'.encode())
-                        with open(self.filename, 'rb') as ifile:
-                            ofile.write(ifile.read())
-                    else:
-                        # OK, I'm a jp2 file.  Need to find out where the
-                        # raw codestream actually starts.
-                        jp2c = [box for box in self.box
-                                if box.box_id == 'jp2c']
-                        jp2c = jp2c[0]
-                        ofile.write(struct.pack('>I', jp2c.length + 8))
-                        ofile.write('jp2c'.encode())
-                        with open(self.filename, 'rb') as ifile:
-                            # Seek 8 bytes past the L, T fields to get to the
-                            # raw codestream.
-                            ifile.seek(jp2c.offset + 8)
-                            ofile.write(ifile.read(jp2c.length - 8))
-
+                    self._write_wrapped_codestream(ofile, box)
             ofile.flush()
 
         jp2 = Jp2k(filename)
         return jp2
 
+    def _write_wrapped_codestream(self, ofile, box):
+        """Write wrapped codestream."""
+        # Codestreams require a bit more care.
+        # Am I a raw codestream?
+        if len(self.box) == 0:
+            # Yes, just write the codestream box header plus all
+            # of myself out to file.
+            ofile.write(struct.pack('>I', self.length + 8))
+            ofile.write(b'jp2c')
+            with open(self.filename, 'rb') as ifile:
+                ofile.write(ifile.read())
+            return
+
+        # OK, I'm a jp2/jpx file.  Need to find out where the raw codestream
+        # actually starts.
+        offset = box.offset
+        if offset == -1:
+            if self.box[1].brand == 'jpx ':
+                msg = "The codestream box must have its offset and "
+                msg += "length attributes fully specified if the file "
+                msg += "type brand is JPX."
+                raise IOError(msg)
+
+            # Find the first codestream in the file.
+            jp2c = [box for box in self.box if box.box_id == 'jp2c']
+            offset = jp2c[0].offset
+
+        # Ready to write the codestream.
+        with open(self.filename, 'rb') as ifile:
+            ifile.seek(offset)
+
+            # Verify that the specified codestream is right.
+            read_buffer = ifile.read(8)
+            L, T = struct.unpack_from('>I4s', read_buffer, 0)
+            if T != b'jp2c':
+                msg = "Unable to locate the specified codestream."
+                raise IOError(msg)
+            if L == 0:
+                # The length of the box is presumed to last until the end of
+                # the file.  Compute the effective length of the box.
+                L = os.path.getsize(ifile.name) - ifile.tell() + 8
+
+            elif L == 1:
+                # The length of the box is in the XL field, a 64-bit value.
+                read_buffer = ifile.read(8)
+                L, = struct.unpack('>Q', read_buffer)
+
+            ifile.seek(offset)
+            read_buffer = ifile.read(L)
+            ofile.write(read_buffer)
+
+    def _get_default_jp2_boxes(self):
+        """Create a default set of JP2 boxes."""
+        # Try to create a reasonable default.
+        boxes = [JPEG2000SignatureBox(),
+                 FileTypeBox(),
+                 JP2HeaderBox(),
+                 ContiguousCodestreamBox()]
+        codestream = self.get_codestream()
+        height = codestream.segment[1].ysiz
+        width = codestream.segment[1].xsiz
+        num_components = len(codestream.segment[1].xrsiz)
+        if num_components < 3:
+            colorspace = GREYSCALE
+        else:
+            if len(self.box) == 0:
+                # Best guess is SRGB
+                colorspace = SRGB
+            else:
+                # Take whatever the first jp2 header / color specification
+                # says.
+                jp2hs = [box for box in self.box if box.box_id == 'jp2h']
+                colorspace = jp2hs[0].box[1].colorspace
+
+        boxes[2].box = [ImageHeaderBox(height=height, width=width,
+                                       num_components=num_components),
+                        ColourSpecificationBox(colorspace=colorspace)]
+
+        return boxes
+
     def read(self, **kwargs):
         """Read a JPEG 2000 image.
 
@@ -636,6 +775,9 @@ class Jp2k(Jp2kBox):
             (first_row, first_col, last_row, last_col)
         tile : int, optional
             Number of tile to decode.
+        ignore_pclr_cmap_cdef : bool
+            Whether or not to ignore the pclr, cmap, or cdef boxes during any
+            color transformation.  Defaults to False.
         verbose : bool, optional
             Print informational messages produced by the OpenJPEG library.
 
@@ -690,7 +832,8 @@ class Jp2k(Jp2kBox):
             msg += "the read_bands method instead."
             raise RuntimeError(msg)
 
-    def _read_openjpeg(self, rlevel=0, verbose=False):
+    def _read_openjpeg(self, rlevel=0, ignore_pclr_cmap_cdef=False,
+                       verbose=False):
         """Read a JPEG 2000 image using libopenjpeg.
 
         Parameters
@@ -698,6 +841,9 @@ class Jp2k(Jp2kBox):
         rlevel : int, optional
             Factor by which to rlevel output resolution.  Use -1 to get the
             lowest resolution thumbnail.
+        ignore_pclr_cmap_cdef : bool
+            Whether or not to ignore the pclr, cmap, or cdef boxes during any
+            color transformation.  Defaults to False.
         verbose : bool, optional
             Print informational messages produced by the OpenJPEG library.
 
@@ -713,32 +859,12 @@ class Jp2k(Jp2kBox):
         """
         self._subsampling_sanity_check()
 
-        if rlevel != 0:
-            # Must check the specified rlevel against the maximum.
-            # OpenJPEG 1.3 will segfault if rlevel is too high.
-            codestream = self.get_codestream()
-            max_rlevel = codestream.segment[2].spcod[4]
-            if rlevel == -1:
-                # -1 is shorthand for the largest rlevel
-                rlevel = max_rlevel
-            if rlevel < -1 or rlevel > max_rlevel:
-                msg = "rlevel must be in the range [-1, {0}] for this image."
-                msg = msg.format(max_rlevel)
-                raise IOError(msg)
+        dparameters = self._populate_dparam(rlevel, ignore_pclr_cmap_cdef)
 
         with ExitStack() as stack:
             try:
-                # Set decoding parameters.
-                dparameters = opj.DecompressionParametersType()
-                opj.set_default_decoder_parameters(ctypes.byref(dparameters))
-                dparameters.cp_reduce = rlevel
                 dparameters.decod_format = self._codec_format
 
-                infile = self.filename.encode()
-                nelts = opj.PATH_LEN - len(infile)
-                infile += b'0' * nelts
-                dparameters.infile = infile
-
                 dinfo = opj.create_decompress(dparameters.decod_format)
 
                 event_mgr = opj.EventMgrType()
@@ -775,7 +901,7 @@ class Jp2k(Jp2kBox):
         return data
 
     def _read_openjp2(self, rlevel=0, layer=0, area=None, tile=None,
-                      verbose=False):
+                      verbose=False, ignore_pclr_cmap_cdef=False):
         """Read a JPEG 2000 image using libopenjp2.
 
         Parameters
@@ -805,19 +931,19 @@ class Jp2k(Jp2kBox):
         """
         self._subsampling_sanity_check()
 
-        dparam = self._populate_dparam(layer, rlevel, area, tile)
+        dparam = self._populate_dparam(rlevel, ignore_pclr_cmap_cdef,
+                                       layer=layer, tile=tile, area=area)
 
         with ExitStack() as stack:
-            if re.match("2.0", version.openjpeg_version):
+            if re.match("2.1", version.openjpeg_version):
+                filename = self.filename
+                stream = opj2.stream_create_default_file_stream(filename, True)
+                stack.callback(opj2.stream_destroy, stream)
+            else:
                 fptr = libc.fopen(self.filename, 'rb')
                 stack.callback(libc.fclose, fptr)
                 stream = opj2.stream_create_default_file_stream(fptr, True)
                 stack.callback(opj2.stream_destroy, stream)
-            else:
-                filename = self.filename
-                stream = opj2.stream_create_default_file_stream(filename, True)
-                stack.callback(opj2.stream_destroy, stream)
-
             codec = opj2.create_decompress(self._codec_format)
             stack.callback(opj2.destroy_codec, codec)
 
@@ -848,27 +974,35 @@ class Jp2k(Jp2kBox):
 
         return img_array
 
-    def _populate_dparam(self, layer, rlevel, area, tile):
+    def _populate_dparam(self, rlevel, ignore_pclr_cmap_cdef, tile=None,
+                         layer=None, area=None):
         """Populate decompression structure with appropriate input parameters.
 
         Parameters
         ----------
-        layer : int, optional
+        layer : int
             Number of quality layer to decode.
-        rlevel : int, optional
+        rlevel : int
             Factor by which to rlevel output resolution.
-        area : tuple, optional
+        area : tuple
             Specifies decoding image area,
             (first_row, first_col, last_row, last_col)
-        tile : int, optional
+        tile : int
             Number of tile to decode.
+        ignore_pclr_cmap_cdef : bool
+            Whether or not to ignore the pclr, cmap, or cdef boxes during any
+            color transformation.  Defaults to False.
 
         Returns
         -------
         dparam : DecompressionParametersType (ctypes)
             Corresponds to openjp2 decompression parameters structure.
         """
-        dparam = opj2.set_default_decoder_parameters()
+        if opj2.OPENJP2 is not None:
+            dparam = opj2.set_default_decoder_parameters()
+        else:
+            dparam = opj.DecompressionParametersType()
+            opj.set_default_decoder_parameters(ctypes.byref(dparam))
 
         infile = self.filename.encode()
         nelts = opj2.PATH_LEN - len(infile)
@@ -877,12 +1011,22 @@ class Jp2k(Jp2kBox):
 
         dparam.decod_format = self._codec_format
 
-        dparam.cp_layer = layer
+        if layer is not None:
+            dparam.cp_layer = layer
 
-        if rlevel == -1:
-            # Get the lowest resolution thumbnail.
+        # Must check the specified rlevel against the maximum.
+        if rlevel != 0:
+            # Must check the specified rlevel against the maximum.
             codestream = self.get_codestream()
-            rlevel = codestream.segment[2].spcod[4]
+            max_rlevel = codestream.segment[2].spcod[4]
+            if rlevel == -1:
+                # -1 is shorthand for the largest rlevel
+                rlevel = max_rlevel
+            elif rlevel < -1 or rlevel > max_rlevel:
+                msg = "rlevel must be in the range [-1, {0}] for this image."
+                msg = msg.format(max_rlevel)
+                raise IOError(msg)
+
         dparam.cp_reduce = rlevel
 
         if area is not None:
@@ -899,10 +1043,14 @@ class Jp2k(Jp2kBox):
             dparam.tile_index = tile
             dparam.nb_tile_to_decode = 1
 
+        if ignore_pclr_cmap_cdef is True:
+            # Return raw codestream components.
+            dparam.flags |= 1
+
         return dparam
 
     def read_bands(self, rlevel=0, layer=0, area=None, tile=None,
-                   verbose=False):
+                   verbose=False, ignore_pclr_cmap_cdef=False):
         """Read a JPEG 2000 image.
 
         The only time you should use this method is when the image has
@@ -920,6 +1068,9 @@ class Jp2k(Jp2kBox):
             (first_row, first_col, last_row, last_col)
         tile : int, optional
             Number of tile to decode.
+        ignore_pclr_cmap_cdef : bool
+            Whether or not to ignore the pclr, cmap, or cdef boxes during any
+            color transformation.  Defaults to False.
         verbose : bool, optional
             Print informational messages produced by the OpenJPEG library.
 
@@ -949,18 +1100,20 @@ class Jp2k(Jp2kBox):
                                        "of OpenJP2 installed before using "
                                        "this functionality.")
 
-        dparam = self._populate_dparam(layer, rlevel, area, tile)
+        dparam = self._populate_dparam(rlevel, ignore_pclr_cmap_cdef,
+                                       layer=layer, tile=tile, area=area)
 
         with ExitStack() as stack:
-            if re.match("2.0", version.openjpeg_version):
+            if re.match("2.1", version.openjpeg_version):
+                # API change in 2.1
+                filename = self.filename
+                stream = opj2.stream_create_default_file_stream(filename, True)
+                stack.callback(opj2.stream_destroy, stream)
+            else:
                 fptr = libc.fopen(self.filename, 'rb')
                 stack.callback(libc.fclose, fptr)
                 stream = opj2.stream_create_default_file_stream(fptr, True)
                 stack.callback(opj2.stream_destroy, stream)
-            else:
-                filename = self.filename
-                stream = opj2.stream_create_default_file_stream(filename, True)
-                stack.callback(opj2.stream_destroy, stream)
             codec = opj2.create_decompress(self._codec_format)
             stack.callback(opj2.destroy_codec, codec)
 
@@ -1008,8 +1161,8 @@ class Jp2k(Jp2kBox):
         >>> jp2 = glymur.Jp2k(jfile)
         >>> codestream = jp2.get_codestream()
         >>> print(codestream.segment[1])
-        SIZ marker segment @ (3137, 47)
-            Profile:  2
+        SIZ marker segment @ (3233, 47)
+            Profile:  no profile
             Reference Grid Height, Width:  (1456 x 2592)
             Vertical, Horizontal Reference Grid Offset:  (0 x 0)
             Reference Tile Height, Width:  (1456 x 2592)
@@ -1017,21 +1170,14 @@ class Jp2k(Jp2kBox):
             Bitdepth:  (8, 8, 8)
             Signed:  (False, False, False)
             Vertical, Horizontal Subsampling:  ((1, 1), (1, 1), (1, 1))
-
-        Raises
-        ------
-        IOError
-            If the file is JPX with more than one codestream.
         """
         with open(self.filename, 'rb') as fptr:
             if self._codec_format == opj2.CODEC_J2K:
                 codestream = Codestream(fptr, self.length,
                                         header_only=header_only)
             else:
+                ftyp = self.box[1]
                 box = [x for x in self.box if x.box_id == 'jp2c']
-                if len(box) != 1:
-                    msg = "JP2 files must have a single codestream."
-                    raise RuntimeError(msg)
                 fptr.seek(box[0].offset)
                 read_buffer = fptr.read(8)
                 (box_length, _) = struct.unpack('>I4s', read_buffer)
@@ -1049,7 +1195,7 @@ class Jp2k(Jp2kBox):
             return codestream
 
 
-def component2dtype(component):
+def _component2dtype(component):
     """Take an OpenJPEG component structure and determine the numpy datatype.
 
     Parameters
@@ -1090,11 +1236,52 @@ def _validate_nonzero_image_size(nrows, ncols, component_index):
         raise IOError(msg)
 
 
+JP2_IDS = ['colr', 'cdef', 'cmap', 'jp2c', 'ftyp', 'ihdr', 'jp2h', 'jP  ',
+           'pclr', 'res ', 'resc', 'resd', 'xml ', 'ulst', 'uinf', 'url ',
+           'uuid']
+
 def _validate_jp2_box_sequence(boxes):
     """Run through series of tests for JP2 box legality.
 
     This is non-exhaustive.
     """
+    _validate_signature_compatibility(boxes)
+    _validate_jp2h(boxes)
+    _validate_jp2c(boxes)
+    if boxes[1].brand == 'jpx ':
+        _validate_jpx_box_sequence(boxes)
+    else:
+        # Validate the JP2 box IDs.
+        count = _collect_box_count(boxes)
+        for box_id in count.keys():
+            if box_id not in JP2_IDS:
+                msg = "The presence of a '{0}' box requires that the file type "
+                msg += "brand be set to 'jpx '."
+                raise IOError(msg.format(box_id))
+
+        _validate_jp2_colr(boxes)
+
+def _validate_jp2_colr(boxes):
+    """
+    Validate JP2 requirements on colour specification boxes.
+    """
+    lst = [box for box in boxes if box.box_id == 'jp2h']
+    jp2h = lst[0]
+    for colr in [box for box in jp2h.box if box.box_id == 'colr']:
+        if colr.approximation != 0:
+            msg = "A JP2 colr box cannot have a non-zero approximation field."
+            raise IOError(msg)
+
+def _validate_jpx_box_sequence(boxes):
+    """Run through series of tests for JPX box legality."""
+    _validate_label(boxes)
+    _validate_jpx_brand(boxes, boxes[1].brand)
+    _validate_jpx_compatibility(boxes, boxes[1].compatibility_list)
+    _validate_singletons(boxes)
+    _validate_top_level(boxes)
+
+def _validate_signature_compatibility(boxes):
+    """Validate the file signature and compatibility status."""
     # Check for a bad sequence of boxes.
     # 1st two boxes must be 'jP  ' and 'ftyp'
     if boxes[0].box_id != 'jP  ' or boxes[1].box_id != 'ftyp':
@@ -1102,6 +1289,14 @@ def _validate_jp2_box_sequence(boxes):
         msg += "must be the file type box."
         raise IOError(msg)
 
+    # The compatibility list must contain at a minimum 'jp2 '.
+    if 'jp2 ' not in boxes[1].compatibility_list:
+        msg = "The ftyp box must contain 'jp2 ' in the compatibility list."
+        raise IOError(msg)
+
+
+def _validate_jp2c(boxes):
+    """Validate the codestream box in relation to other boxes."""
     # jp2c must be preceeded by jp2h
     jp2h_lst = [idx for (idx, box) in enumerate(boxes)
                 if box.box_id == 'jp2h']
@@ -1118,8 +1313,20 @@ def _validate_jp2_box_sequence(boxes):
         msg = "The codestream box must be preceeded by a jp2 header box."
         raise IOError(msg)
 
+
+def _validate_jp2h(boxes):
+    """Validate the JP2 Header box."""
+    _check_jp2h_child_boxes(boxes, 'top-level')
+
+    jp2h_lst = [box for box in boxes if box.box_id == 'jp2h']
+    jp2h = jp2h_lst[0]
+
+    # 1st jp2 header box cannot be empty.
+    if len(jp2h.box) == 0:
+        msg = "The JP2 header superbox cannot be empty."
+        raise IOError(msg)
+
     # 1st jp2 header box must be ihdr
-    jp2h = boxes[jp2h_idx]
     if jp2h.box[0].box_id != 'ihdr':
         msg = "The first box in the jp2 header box must be the image "
         msg += "header box."
@@ -1133,15 +1340,12 @@ def _validate_jp2_box_sequence(boxes):
         raise IOError(msg)
     colr = jp2h.box[colr_lst[0]]
 
-    # Any cdef box must be in the jp2 header following the image header.
-    cdef_lst = [j for (j, box) in enumerate(boxes) if box.box_id == 'cdef']
-    if len(cdef_lst) != 0:
-        msg = "Any channel defintion box must be in the JP2 header "
-        msg += "following the image header."
-        raise IOError(msg)
+    _validate_channel_definition(jp2h, colr)
 
-    cdef_lst = [j for (j, box) in enumerate(jp2h.box)
-                if box.box_id == 'cdef']
+
+def _validate_channel_definition(jp2h, colr):
+    """Validate the channel definition box."""
+    cdef_lst = [j for (j, box) in enumerate(jp2h.box) if box.box_id == 'cdef']
     if len(cdef_lst) > 1:
         msg = "Only one channel definition box is allowed in the "
         msg += "JP2 header."
@@ -1162,12 +1366,127 @@ def _validate_jp2_box_sequence(boxes):
                 raise IOError(msg)
 
 
+JP2H_CHILDREN = set(['bpcc', 'cdef', 'cmap', 'ihdr', 'pclr'])
+def _check_jp2h_child_boxes(boxes, parent_box_name):
+    """Certain boxes can only reside in the JP2 header."""
+    box_ids = set([box.box_id for box in boxes])
+    intersection = box_ids.intersection(JP2H_CHILDREN)
+    if len(intersection) > 0 and parent_box_name not in ['jp2h', 'jpch']:
+        msg = "A '{0}' box can only be nested in a JP2 header box."
+        raise IOError(msg.format(list(intersection)[0]))
+
+    # Recursively check any contained superboxes.
+    for box in boxes:
+        if hasattr(box, 'box'):
+            _check_jp2h_child_boxes(box.box, box.box_id)
+
+
+def _collect_box_count(boxes):
+    """Count the occurences of each box type."""
+    count = Counter([box.box_id for box in boxes])
+
+    # Add the counts in the superboxes.
+    for box in boxes:
+        if hasattr(box, 'box'):
+            count.update(_collect_box_count(box.box))
+
+    return count
+
+TOP_LEVEL_ONLY_BOXES = set(['dtbl'])
+
+def _check_superbox_for_top_levels(boxes):
+    """Several boxes can only occur at the top level."""
+    # We are only looking at the boxes contained in a superbox, so if any of
+    # the blacklisted boxes show up here, it's an error.
+    box_ids = set([box.box_id for box in boxes])
+    intersection = box_ids.intersection(TOP_LEVEL_ONLY_BOXES)
+    if len(intersection) > 0:
+        msg = "A '{0}' box cannot be nested in a superbox."
+        raise IOError(msg.format(list(intersection)[0]))
+
+    # Recursively check any contained superboxes.
+    for box in boxes:
+        if hasattr(box, 'box'):
+            _check_superbox_for_top_levels(box.box)
+
+def _validate_top_level(boxes):
+    """Several boxes can only occur at the top level."""
+    # Add the counts in the superboxes.
+    for box in boxes:
+        if hasattr(box, 'box'):
+            _check_superbox_for_top_levels(box.box)
+
+    count = _collect_box_count(boxes)
+    # Which boxes occur more than once?
+    multiples = [box_id for box_id, bcount in count.items() if bcount > 1]
+    if 'dtbl' in multiples:
+        raise IOError('There can only be one dtbl box in a file.')
+
+    # If there is one data reference box, then there must also be one ftbl.
+    if 'dtbl' in count and 'ftbl' not in count:
+        msg = 'The presence of a data reference box requires the presence of '
+        msg += 'a fragment table box as well.'
+        raise IOError(msg)
+
+def _validate_singletons(boxes):
+    """Several boxes can only occur once."""
+    count = _collect_box_count(boxes)
+    # Which boxes occur more than once?
+    multiples = [box_id for box_id, bcount in count.items() if bcount > 1]
+    if 'dtbl' in multiples:
+        raise IOError('There can only be one dtbl box in a file.')
+
+def _validate_jpx_brand(boxes, brand):
+    """
+    If there is a JPX box then the brand must be 'jpx '.
+    """
+    for box in boxes:
+        if box.box_id in JPX_IDS:
+            if brand != 'jpx ':
+                msg = "A JPX box requires that the file type box brand be "
+                msg += "'jpx '."
+                raise RuntimeError(msg)
+        if hasattr(box, 'box') != 0:
+            # Same set of checks on any child boxes.
+            _validate_jpx_brand(box.box, brand)
+
+def _validate_jpx_compatibility(boxes, compatibility_list):
+    """
+    If there is a JPX box then the compatibility list must also contain 'jpx '.
+    """
+    jpx_cl = set(compatibility_list)
+    for box in boxes:
+        if box.box_id in JPX_IDS:
+            if len(set(['jpx ', 'jpxb']).intersection(jpx_cl)) == 0:
+                msg = "A JPX box requires that either 'jpx ' or 'jpxb' be "
+                msg += "present in the ftype compatibility list."
+                raise RuntimeError(msg)
+        if hasattr(box, 'box') != 0:
+            # Same set of checks on any child boxes.
+            _validate_jpx_compatibility(box.box, compatibility_list)
+
+def _validate_label(boxes):
+    """
+    Label boxes can only be inside association, codestream headers, or
+    compositing layer header boxes.
+    """
+    for box in boxes:
+        if box.box_id != 'asoc':
+            if hasattr(box, 'box'):
+                for boxi in box.box:
+                    if boxi.box_id == 'lbl ':
+                        msg = "A label box cannot be nested inside a {0} box."
+                        msg = msg.format(box.box_id)
+                        raise IOError(msg)
+                # Same set of checks on any child boxes.
+                _validate_label(box.box)
+
 def extract_image_cube(image):
     """Extract 3D image from openjpeg data structure.
     """
     ncomps = image.contents.numcomps
     component = image.contents.comps[0]
-    dtype = component2dtype(component)
+    dtype = _component2dtype(component)
 
     nrows = component.h
     ncols = component.w
@@ -1201,7 +1520,7 @@ def extract_image_bands(image):
     for k in range(image.contents.numcomps):
         component = image.contents.comps[k]
 
-        dtype = component2dtype(component)
+        dtype = _component2dtype(component)
         nrows = component.h
         ncols = component.w
 
@@ -1328,6 +1647,18 @@ def _populate_image_struct(cparams, image, imgdata):
 
     # Stage the image data to the openjpeg data structure.
     for k in range(0, num_comps):
+        if re.match("2.0", version.openjpeg_version) is not None:
+            # 2.0 API
+            if cparams.cp_cinema:
+                image.contents.comps[k].prec = 12
+                image.contents.comps[k].bpp = 12
+        else:
+            # 2.1 API
+            if cparams.rsiz in (core.OPJ_PROFILE_CINEMA_2K,
+                                core.OPJ_PROFILE_CINEMA_4K):
+                image.contents.comps[k].prec = 12
+                image.contents.comps[k].bpp = 12
+
         layer = np.ascontiguousarray(imgdata[:, :, k], dtype=np.int32)
         dest = image.contents.comps[k].data
         src = layer.ctypes.data
@@ -1387,7 +1718,7 @@ def _validate_compression_params(img_array, cparams):
         msg = "{0}D imagery is not allowed.".format(img_array.ndim)
         raise IOError(msg)
 
-    if re.match("2.0.0", version.openjpeg_version):
+    if re.match("2.0.0", version.openjpeg_version) is not None:
         if (((img_array.ndim != 2) and
              (img_array.shape[2] != 1 and img_array.shape[2] != 3))):
             msg = "Writing images is restricted to single-channel "
