@@ -1,37 +1,28 @@
 """
-Tests for libopenjp2 wrapping functions.  
+Tests for libopenjp2 wrapping functions.
 """
 # R0904:  Seems like pylint is fooled in this situation
 # W0142:  using kwargs is ok in this context
 # pylint: disable=R0904,W0142
 
-# unittest2 is python-2.6 only (pylint/python-2.7)
-# pylint: disable=F0401
-
 import os
 import re
 import sys
 import tempfile
-
-if sys.hexversion < 0x02070000:
-    import unittest2 as unittest
-else:
-    import unittest
+import unittest
 
 import numpy as np
 
 import glymur
 from glymur.lib import openjp2
 
-if re.match("2.0", glymur.version.openjpeg_version):
-    OPENJP2_IS_V2_OFFICIAL = True
-else:
-    OPENJP2_IS_V2_OFFICIAL = False
 
 @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
 @unittest.skipIf(openjp2.OPENJP2 is None,
                  "Missing openjp2 library.")
-@unittest.skipIf(OPENJP2_IS_V2_OFFICIAL, "API followed here specific to V2.1")
+@unittest.skipIf(re.match(r'''(1|2.0)''',
+                              glymur.version.openjpeg_version) is not None,
+                     "Not to be run until 2.1.0")
 class TestOpenJP2(unittest.TestCase):
     """Test openjp2 library functionality.
 
@@ -211,7 +202,7 @@ class TestOpenJP2(unittest.TestCase):
         """Runs test designated tte3 in OpenJPEG test suite."""
         with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:
             xtx3_setup(tfile.name)
-        self.assertTrue(True)            
+        self.assertTrue(True)
 
     def test_rta3(self):
         """Runs test designated rta3 in OpenJPEG test suite."""
@@ -220,13 +211,13 @@ class TestOpenJP2(unittest.TestCase):
 
             codec_format = openjp2.CODEC_J2K
             self.j2k_random_tile_access(tfile.name, codec_format)
-        self.assertTrue(True)            
+        self.assertTrue(True)
 
     def test_tte4(self):
         """Runs test designated tte4 in OpenJPEG test suite."""
         with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:
             xtx4_setup(tfile.name)
-        self.assertTrue(True)            
+        self.assertTrue(True)
 
     def test_rta4(self):
         """Runs test designated rta4 in OpenJPEG test suite."""
@@ -240,7 +231,7 @@ class TestOpenJP2(unittest.TestCase):
         """Runs test designated tte5 in OpenJPEG test suite."""
         with tempfile.NamedTemporaryFile(suffix=".j2k") as tfile:
             xtx5_setup(tfile.name)
-        self.assertTrue(True)            
+        self.assertTrue(True)
 
     def test_rta5(self):
         """Runs test designated rta5 in OpenJPEG test suite."""
@@ -318,7 +309,7 @@ def tile_encoder(**kwargs):
     openjp2.setup_encoder(codec, l_param, l_image)
 
     stream = openjp2.stream_create_default_file_stream(kwargs['filename'],
-                                                       False)
+                                                          False)
     openjp2.start_compress(codec, l_image, stream)
 
     for j in np.arange(num_tiles):
@@ -331,11 +322,11 @@ def tile_encoder(**kwargs):
 
 def tile_decoder(**kwargs):
     """Fixture called with various configurations by many tests.
-    
-    Reads a tile.  That's all it does. 
+
+    Reads a tile.  That's all it does.
     """
     stream = openjp2.stream_create_default_file_stream(kwargs['filename'],
-                                                       True)
+                                                          True)
     dparam = openjp2.set_default_decoder_parameters()
 
     dparam.decod_format = kwargs['codec_format']
@@ -354,7 +345,7 @@ def tile_decoder(**kwargs):
 
     openjp2.setup_decoder(codec, dparam)
     image = openjp2.read_header(stream, codec)
-    openjp2.set_decode_area(codec, image, 
+    openjp2.set_decode_area(codec, image,
                             kwargs['x0'], kwargs['y0'],
                             kwargs['x1'], kwargs['y1'])
 
