@@ -13,10 +13,7 @@ import sys
 import tempfile
 import warnings
 
-if sys.hexversion < 0x02070000:
-    import unittest2 as unittest
-else:
-    import unittest
+import unittest
 
 if sys.hexversion <= 0x03030000:
     from mock import patch
