@@ -8,36 +8,43 @@ Test suite for codestream parsing.
 # tempfile.TemporaryDirectory, unittest.assertWarns introduced in 3.2
 # pylint: disable=E1101
 
-# unittest2 is python2.6 only (pylint/python-2.7)
-# pylint: disable=F0401
-
 import os
 import struct
 import sys
 import tempfile
+import unittest
 import warnings
 
-if sys.hexversion < 0x02070000:
-    import unittest2 as unittest
-else:
-    import unittest
-
 from glymur import Jp2k
 import glymur
 
-try:
-    DATA_ROOT = os.environ['OPJ_DATA_ROOT']
-except KeyError:
-    DATA_ROOT = None
-except:
-    raise
+from .fixtures import opj_data_file, OPJ_DATA_ROOT
 
-
-@unittest.skipIf(DATA_ROOT is None,
-                 "OPJ_DATA_ROOT environment variable not set")
 class TestCodestream(unittest.TestCase):
     """Test suite for unusual codestream cases."""
 
+    def setUp(self):
+        self.jp2file = glymur.data.nemo()
+
+    def tearDown(self):
+        pass
+
+    def test_siz_segment_ssiz_unsigned(self):
+        """ssiz attribute to be removed in future release"""
+        j = Jp2k(self.jp2file)
+        codestream = j.get_codestream()
+
+        # The ssiz attribute was simply a tuple of raw bytes.
+        # The first 7 bits are interpreted as the bitdepth, the MSB determines
+        # whether or not it is signed.
+        self.assertEqual(codestream.segment[1].ssiz, (7, 7, 7))
+
+
+@unittest.skipIf(OPJ_DATA_ROOT is None,
+                 "OPJ_DATA_ROOT environment variable not set")
+class TestCodestreamOpjData(unittest.TestCase):
+    """Test suite for unusual codestream cases.  Uses OPJ_DATA_ROOT"""
+
     def setUp(self):
         self.jp2file = glymur.data.nemo()
 
@@ -53,7 +60,7 @@ class TestCodestream(unittest.TestCase):
         #
         # Let's inject a reserved marker segment into a file that
         # we know something about to make sure we can still parse it.
-        filename = os.path.join(DATA_ROOT, 'input/conformance/p0_01.j2k')
+        filename = os.path.join(OPJ_DATA_ROOT, 'input/conformance/p0_01.j2k')
         with tempfile.NamedTemporaryFile(suffix='.j2k') as tfile:
             with open(filename, 'rb') as ifile:
                 # Everything up until the first QCD marker.
@@ -75,42 +82,9 @@ class TestCodestream(unittest.TestCase):
             self.assertEqual(codestream.segment[2].length, 3)
             self.assertEqual(codestream.segment[2].data, b'\x00')
 
-    @unittest.skipIf(sys.hexversion < 0x03020000,
-                     "Uses features introduced in 3.2.")
-    @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
-    def test_unknown_marker_segment(self):
-        """Should warn for an unknown marker."""
-        # Let's inject a marker segment whose marker does not appear to
-        # be valid.  We still parse the file, but warn about the offending
-        # marker.
-        filename = os.path.join(DATA_ROOT, 'input/conformance/p0_01.j2k')
-        with tempfile.NamedTemporaryFile(suffix='.j2k') as tfile:
-            with open(filename, 'rb') as ifile:
-                # Everything up until the first QCD marker.
-                read_buffer = ifile.read(45)
-                tfile.write(read_buffer)
-
-                # Write the new marker segment, 0xff79 = 65401
-                read_buffer = struct.pack('>HHB', int(65401), int(3), int(0))
-                tfile.write(read_buffer)
-
-                # Get the rest of the input file.
-                read_buffer = ifile.read()
-                tfile.write(read_buffer)
-                tfile.flush()
-
-                with warnings.catch_warnings(record=True) as w:
-                    warnings.simplefilter("always")
-                    codestream = Jp2k(tfile.name).get_codestream()
-                    self.assertEqual(len(w), 1)
-
-                self.assertEqual(codestream.segment[2].marker_id, '0xff79')
-                self.assertEqual(codestream.segment[2].length, 3)
-                self.assertEqual(codestream.segment[2].data, b'\x00')
-
     def test_psot_is_zero(self):
         """Psot=0 in SOT is perfectly legal.  Issue #78."""
-        filename = os.path.join(DATA_ROOT,
+        filename = os.path.join(OPJ_DATA_ROOT,
                                 'input/nonregression/123.j2c')
         j = Jp2k(filename)
         codestream = j.get_codestream(header_only=False)
@@ -119,5 +93,61 @@ class TestCodestream(unittest.TestCase):
         # codestream, so the last one is EOC.
         self.assertEqual(codestream.segment[-1].marker_id, 'EOC')
 
+
+    def test_siz_segment_ssiz_signed(self):
+        """ssiz attribute to be removed in future release"""
+        filename = os.path.join(OPJ_DATA_ROOT, 'input/conformance/p0_03.j2k')
+        j = Jp2k(filename)
+        codestream = j.get_codestream()
+
+        # The ssiz attribute was simply a tuple of raw bytes.
+        # The first 7 bits are interpreted as the bitdepth, the MSB determines
+        # whether or not it is signed.
+        self.assertEqual(codestream.segment[1].ssiz, (131,))
+
+
+class TestCodestreamRepr(unittest.TestCase):
+
+    def setUp(self):
+        self.jp2file = glymur.data.nemo()
+
+    def tearDown(self):
+        pass
+
+    def test_soc(self):
+        """Test SOC segment repr"""
+        segment = glymur.codestream.SOCsegment()
+        newseg = eval(repr(segment))
+        self.assertEqual(newseg.marker_id, 'SOC')
+
+    def test_siz(self):
+        """Test SIZ segment repr"""
+        kwargs = {'rsiz': 0,
+                  'xysiz': (2592, 1456),
+                  'xyosiz': (0, 0),
+                  'xytsiz': (2592, 1456),
+                  'xytosiz': (0, 0),
+                  'Csiz': 3,
+                  'bitdepth': (8, 8, 8),
+                  'signed':  (False, False, False),
+                  'xyrsiz': ((1, 1, 1), (1, 1, 1))}
+        segment = glymur.codestream.SIZsegment(**kwargs)
+        newseg = eval(repr(segment))
+        self.assertEqual(newseg.marker_id, 'SIZ')
+        self.assertEqual(newseg.xsiz, 2592)
+        self.assertEqual(newseg.ysiz, 1456)
+        self.assertEqual(newseg.xosiz, 0)
+        self.assertEqual(newseg.yosiz, 0)
+        self.assertEqual(newseg.xtsiz, 2592)
+        self.assertEqual(newseg.ytsiz, 1456)
+        self.assertEqual(newseg.xtosiz, 0)
+        self.assertEqual(newseg.ytosiz, 0)
+
+        self.assertEqual(newseg.xrsiz, (1, 1, 1))
+        self.assertEqual(newseg.yrsiz, (1, 1, 1))
+        self.assertEqual(newseg.bitdepth, (8, 8, 8))
+        self.assertEqual(newseg.signed, (False, False, False))
+
+
 if __name__ == "__main__":
     unittest.main()
