@@ -15,13 +15,9 @@ import imp
 import os
 import sys
 import tempfile
+import unittest
 import warnings
 
-if sys.hexversion < 0x02070000:
-    import unittest2 as unittest
-else:
-    import unittest
-
 if sys.hexversion <= 0x03030000:
     from mock import patch
 else:
@@ -89,9 +85,10 @@ class TestSuite(unittest.TestCase):
                         # Misconfigured new configuration file should
                         # be rejected.
                         with warnings.catch_warnings(record=True) as w:
-                            warnings.simplefilter("always")
+                            warnings.simplefilter('always')
                             imp.reload(glymur.lib.openjp2)
-                            self.assertEqual(len(w), 1)
+                            self.assertTrue(issubclass(w[0].category,UserWarning))
+                            self.assertTrue('could not be loaded' in str(w[0].message))
 
 
 @unittest.skipIf(glymur.lib.openjp2.OPENJP2 is None and
