@@ -1,6 +1,9 @@
 """
 Tests for general glymur functionality.
 """
+# E1101:  assertWarns introduced in python 3.2
+# pylint: disable=E1101
+
 # R0904:  Not too many methods in unittest.
 # pylint: disable=R0904
 
@@ -14,14 +17,10 @@ import shutil
 import struct
 import sys
 import tempfile
+import unittest
 import uuid
 from xml.etree import cElementTree as ET
 
-if sys.hexversion < 0x02070000:
-    import unittest2 as unittest
-else:
-    import unittest
-
 import warnings
 
 import numpy as np
@@ -30,9 +29,13 @@ import pkg_resources
 import glymur
 from glymur import Jp2k
 
-from .fixtures import OPENJP2_IS_V2_OFFICIAL
-from .fixtures import OPJ_DATA_ROOT, opj_data_file
+from .fixtures import HAS_PYTHON_XMP_TOOLKIT
+if HAS_PYTHON_XMP_TOOLKIT:
+    import libxmp
+    from libxmp import XMPMeta
 
+from .fixtures import OPJ_DATA_ROOT, opj_data_file
+from . import fixtures
 
 # Doc tests should be run as well.
 def load_tests(loader, tests, ignore):
@@ -44,26 +47,69 @@ def load_tests(loader, tests, ignore):
     if os.name == "nt":
         # Can't do it on windows, temporary file issue.
         return tests
-    if sys.hexversion < 0x02070000:
-        # Don't bother with doctests on 2.6 for the time being.
-        return tests
     if glymur.lib.openjp2.OPENJP2 is not None:
         tests.addTests(doctest.DocTestSuite('glymur.jp2k'))
     return tests
 
 
 class TestJp2k(unittest.TestCase):
-    """Test suite for openjpeg software starting at 1.3"""
-
-    # These tests should be run by just about all configuration.
+    """These tests should be run by just about all configuration."""
 
     def setUp(self):
         self.jp2file = glymur.data.nemo()
         self.j2kfile = glymur.data.goodstuff()
+        self.jpxfile = glymur.data.jpxfile()
 
     def tearDown(self):
         pass
 
+    @unittest.skipIf(os.name == "nt", "Unexplained failure on windows")
+    def test_irreversible(self):
+        """Irreversible"""
+        j = Jp2k(self.jp2file)
+        expdata = j.read()
+        with tempfile.NamedTemporaryFile(suffix='.j2k') as tfile:
+            j2 = Jp2k(tfile.name, 'wb')
+            j2.write(expdata, irreversible=True)
+
+            codestream = j2.get_codestream()
+            self.assertEqual(codestream.segment[2].spcod[8],
+                             glymur.core.WAVELET_XFORM_9X7_IRREVERSIBLE)
+
+            actdata = j2.read()
+            self.assertTrue(fixtures.mse(actdata[0], expdata[0]) < 0.38)
+
+    @unittest.skipIf(re.match('1.5.(1|2)',
+                              glymur.version.openjpeg_version) is not None,
+                     "Mysteriously fails in 1.5.1 and 1.5.2")
+    def test_no_cxform_pclr_jpx(self):
+        """Indices for pclr jpxfile if no color transform"""
+        j = Jp2k(self.jpxfile)
+        rgb = j.read()
+        idx = j.read(ignore_pclr_cmap_cdef=True)
+        nr, nc = 1024, 1024
+        self.assertEqual(rgb.shape, (nr, nc, 3))
+        self.assertEqual(idx.shape, (nr, nc))
+
+        # Should be able to manually reconstruct the RGB image from the palette
+        # and indices.
+        palette = j.box[2].box[2].palette
+        rgb_from_idx = np.zeros(rgb.shape, dtype=np.uint8)
+        for r in np.arange(nr):
+            for c in np.arange(nc):
+                rgb_from_idx[r, c] = palette[idx[r, c]]
+        np.testing.assert_array_equal(rgb, rgb_from_idx)
+
+    @unittest.skipIf(os.name == "nt", "Unexplained failure on windows")
+    def test_repr(self):
+        """Verify that results of __repr__ are eval-able."""
+        j = Jp2k(self.j2kfile)
+        newjp2 = eval(repr(j))
+
+        self.assertEqual(newjp2.filename, self.j2kfile)
+        self.assertEqual(newjp2.mode, 'rb')
+        self.assertEqual(len(newjp2.box), 0)
+
     def test_rlevel_max(self):
         """Verify that rlevel=-1 gets us the lowest resolution image"""
         j = Jp2k(self.j2kfile)
@@ -97,7 +143,7 @@ class TestJp2k(unittest.TestCase):
         jp2k = Jp2k(self.jp2file)
 
         # top-level boxes
-        self.assertEqual(len(jp2k.box), 6)
+        self.assertEqual(len(jp2k.box), 5)
 
         self.assertEqual(jp2k.box[0].box_id, 'jP  ')
         self.assertEqual(jp2k.box[0].offset, 0)
@@ -116,15 +162,11 @@ class TestJp2k(unittest.TestCase):
 
         self.assertEqual(jp2k.box[3].box_id, 'uuid')
         self.assertEqual(jp2k.box[3].offset, 77)
-        self.assertEqual(jp2k.box[3].length, 638)
+        self.assertEqual(jp2k.box[3].length, 3146)
 
-        self.assertEqual(jp2k.box[4].box_id, 'uuid')
-        self.assertEqual(jp2k.box[4].offset, 715)
-        self.assertEqual(jp2k.box[4].length, 2412)
-
-        self.assertEqual(jp2k.box[5].box_id, 'jp2c')
-        self.assertEqual(jp2k.box[5].offset, 3127)
-        self.assertEqual(jp2k.box[5].length, 1132296)
+        self.assertEqual(jp2k.box[4].box_id, 'jp2c')
+        self.assertEqual(jp2k.box[4].offset, 3223)
+        self.assertEqual(jp2k.box[4].length, 1132296)
 
         # jp2h super box
         self.assertEqual(len(jp2k.box[2].box), 2)
@@ -166,7 +208,7 @@ class TestJp2k(unittest.TestCase):
         with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
             with open(self.jp2file, 'rb') as ifile:
                 # Everything up until the jp2c box.
-                write_buffer = ifile.read(3127)
+                write_buffer = ifile.read(3223)
                 tfile.write(write_buffer)
 
                 # The L field must be 1 in order to signal the presence of the
@@ -187,9 +229,9 @@ class TestJp2k(unittest.TestCase):
 
             jp2k = Jp2k(tfile.name)
 
-            self.assertEqual(jp2k.box[5].box_id, 'jp2c')
-            self.assertEqual(jp2k.box[5].offset, 3127)
-            self.assertEqual(jp2k.box[5].length, 1133427 + 8)
+            self.assertEqual(jp2k.box[4].box_id, 'jp2c')
+            self.assertEqual(jp2k.box[4].offset, 3223)
+            self.assertEqual(jp2k.box[4].length, 1133427 + 8)
 
     @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
     def test_length_field_is_zero(self):
@@ -241,19 +283,6 @@ class TestJp2k(unittest.TestCase):
         j2k = Jp2k(self.j2kfile)
         j2k.read()
 
-    @unittest.skipIf(OPJ_DATA_ROOT is None,
-                     "OPJ_DATA_ROOT environment variable not set")
-    def test_read_differing_subsamples(self):
-        """should error out with read used on differently subsampled images"""
-        # Verify that we error out appropriately if we use the read method
-        # on an image with differing subsamples
-        #
-        # Issue 86.
-        filename = opj_data_file('input/conformance/p0_05.j2k')
-        j = Jp2k(filename)
-        with self.assertRaises(RuntimeError):
-            j.read()
-
     def test_empty_box_with_j2k(self):
         """Verify that the list of boxes in a J2C/J2K file is present, but
         empty.
@@ -351,48 +380,37 @@ class TestJp2k(unittest.TestCase):
             self.assertEqual(ET.tostring(jp2k.box[3].xml.getroot()),
                              b'<test>this is a test</test>')
 
+    @unittest.skipIf(not HAS_PYTHON_XMP_TOOLKIT, 
+                     "Requires Python XMP Toolkit >= 2.0")
     def test_xmp_attribute(self):
         """Verify the XMP packet in the shipping example file can be read."""
         j = Jp2k(self.jp2file)
-        xmp = j.box[4].data
+
+        xmp = j.box[3].data
         ns0 = '{http://www.w3.org/1999/02/22-rdf-syntax-ns#}'
         ns1 = '{http://ns.adobe.com/xap/1.0/}'
-        name = '{0}RDF/{0}Description'.format(ns0)
+        name = '{0}RDF/{0}Description/{1}CreatorTool'.format(ns0, ns1)
         elt = xmp.find(name)
-        attr_value = elt.attrib['{0}CreatorTool'.format(ns1)]
-        self.assertEqual(attr_value, 'glymur')
+        self.assertEqual(elt.text, 'Google')
 
-    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
-    def test_unrecognized_exif_tag(self):
-        """An unrecognized exif tag should be handled gracefully."""
-        with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
-            shutil.copyfile(self.jp2file, tfile.name)
+        xmp = XMPMeta()
+        xmp.parse_from_str(j.box[3].raw_data.decode('utf-8'),
+                           xmpmeta_wrap=False)
+        creator_tool = xmp.get_property(libxmp.consts.XMP_NS_XMP, 'CreatorTool')
+        self.assertEqual(creator_tool, 'Google') 
 
-            # The Exif UUID starts at byte 77.  There are 8 bytes for the L and
-            # T fields, then 16 bytes for the UUID identifier, then 6 exif
-            # header bytes, then 8 bytes for the TIFF header, then 2 bytes
-            # the the Image IFD number of tags, where we finally find the first
-            # tag, "Make" (271).  We'll corrupt it by changing it into 171,
-            # which does not correspond to any known Exif Image tag.
-            with open(tfile.name, 'r+b') as fptr:
-                fptr.seek(117)
-                write_buffer = struct.pack('<H', int(171))
-                fptr.write(write_buffer)
-
-            # Verify that a warning is issued, but only on python3.
-            # On python2, just suppress the warning.
-            with warnings.catch_warnings():
-                warnings.simplefilter("ignore")
-                j = Jp2k(tfile.name)
-
-            exif = j.box[3].data
-            # Were the tag == 271, 'Make' would be in the keys instead.
-            self.assertTrue(171 in exif['Image'].keys())
-            self.assertFalse('Make' in exif['Image'].keys())
+    @unittest.skipIf(re.match(r'''(1|2.0.0)''',
+                              glymur.version.openjpeg_version) is not None,
+                     "Not supported until 2.0.1")
+    def test_jpx_mult_codestreams_jp2_brand(self):
+        """Read JPX codestream when jp2-compatible."""
+        # The file in question has multiple codestreams.
+        jpx = Jp2k(self.jpxfile)
+        data = jpx.read()
+        self.assertEqual(data.shape, (1024, 1024, 3))
 
 
-@unittest.skipIf(re.match(r"""1\.[01234]""", glymur.version.openjpeg_version),
-                 "Requires at least version 1.5")
+@unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
 class TestJp2k_write(unittest.TestCase):
     """Write tests, can be run by versions 1.5+"""
 
@@ -403,8 +421,6 @@ class TestJp2k_write(unittest.TestCase):
     def tearDown(self):
         pass
 
-
-    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
     def test_cblkh_different_than_width(self):
         """Verify that we can set a code block size where height does not equal
         width.
@@ -421,7 +437,6 @@ class TestJp2k_write(unittest.TestCase):
             # Code block size is reported as XY in the codestream.
             self.assertEqual(tuple(codestream.segment[2].spcod[5:7]), (3, 2))
 
-    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
     def test_too_many_dimensions(self):
         """OpenJP2 only allows 2D or 3D images."""
         with tempfile.NamedTemporaryFile(suffix='.j2k') as tfile:
@@ -430,7 +445,6 @@ class TestJp2k_write(unittest.TestCase):
                 data = np.zeros((128, 128, 2, 2), dtype=np.uint8)
                 j.write(data)
 
-    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
     def test_2d_rgb(self):
         """RGB must have at least 3 components."""
         with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
@@ -439,7 +453,6 @@ class TestJp2k_write(unittest.TestCase):
                 data = np.zeros((128, 128, 2), dtype=np.uint8)
                 j.write(data, colorspace='rgb')
 
-    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
     def test_colorspace_with_j2k(self):
         """Specifying a colorspace with J2K does not make sense"""
         with tempfile.NamedTemporaryFile(suffix='.j2k') as tfile:
@@ -448,7 +461,6 @@ class TestJp2k_write(unittest.TestCase):
                 data = np.zeros((128, 128, 3), dtype=np.uint8)
                 j.write(data, colorspace='rgb')
 
-    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
     def test_specify_rgb(self):
         """specify RGB explicitly"""
         with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
@@ -457,7 +469,6 @@ class TestJp2k_write(unittest.TestCase):
             j.write(data, colorspace='rgb')
             self.assertEqual(j.box[2].box[1].colorspace, glymur.core.SRGB)
 
-    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
     def test_specify_gray(self):
         """test gray explicitly specified (that's GRAY, not GREY)"""
         with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
@@ -467,7 +478,6 @@ class TestJp2k_write(unittest.TestCase):
             self.assertEqual(j.box[2].box[1].colorspace,
                              glymur.core.GREYSCALE)
 
-    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
     def test_specify_grey(self):
         """test grey explicitly specified"""
         with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
@@ -477,7 +487,6 @@ class TestJp2k_write(unittest.TestCase):
             self.assertEqual(j.box[2].box[1].colorspace,
                              glymur.core.GREYSCALE)
 
-    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
     def test_grey_with_two_extra_comps(self):
         """should be able to write gray + two extra components"""
         with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
@@ -498,7 +507,6 @@ class TestJp2k_write(unittest.TestCase):
                 data = np.zeros((128, 128, 3), dtype=np.uint8)
                 j.write(data, colorspace='ycc')
 
-    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
     def test_write_with_jp2_in_caps(self):
         """should be able to write with JP2 suffix."""
         j2k = Jp2k(self.j2kfile)
@@ -509,7 +517,6 @@ class TestJp2k_write(unittest.TestCase):
             actdata = ofile.read()
             np.testing.assert_array_equal(actdata, expdata)
 
-    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
     def test_write_srgb_without_mct(self):
         """should be able to write RGB without specifying mct"""
         j2k = Jp2k(self.j2kfile)
@@ -523,7 +530,6 @@ class TestJp2k_write(unittest.TestCase):
             codestream = ofile.get_codestream()
             self.assertEqual(codestream.segment[2].spcod[3], 0)  # no mct
 
-    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
     def test_write_grayscale_with_mct(self):
         """MCT usage makes no sense for grayscale images."""
         j2k = Jp2k(self.j2kfile)
@@ -533,7 +539,6 @@ class TestJp2k_write(unittest.TestCase):
             with self.assertRaises(IOError):
                 ofile.write(expdata[:, :, 0], mct=True)
 
-    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
     def test_write_cprl(self):
         """Must be able to write a CPRL progression order file"""
         # Issue 17
@@ -583,8 +588,9 @@ class TestJp2k_1_x(unittest.TestCase):
             j2k.read(layer=1)
 
 
-@unittest.skipIf(re.match("2.0.0", glymur.version.openjpeg_version) is None,
-                 "Tests only to be run on 2.0.0.")
+@unittest.skipIf(re.match(r'''2.0.0''',
+                          glymur.version.openjpeg_version) is None,
+                 "Tests only to be run on 2.0 official.")
 class TestJp2k_2_0_official(unittest.TestCase):
     """Test suite to only be run on v2.0 official."""
 
@@ -681,9 +687,9 @@ class TestJp2k_2_0(unittest.TestCase):
                 self.assertEqual(jasoc.box[3].box[1].box_id, 'xml ')
 
 
-@unittest.skipIf(glymur.version.openjpeg_version_tuple[0] < 2 or
-                 OPENJP2_IS_V2_OFFICIAL,
-                 "Missing openjp2 library version 2.0+.")
+@unittest.skipIf(re.match(r'''(1|2.0.0)''',
+                          glymur.version.openjpeg_version) is not None,
+                 "Not to be run until unless 2.0.1 or higher is present")
 class TestJp2k_2_1(unittest.TestCase):
     """Only to be run in 2.0+."""
 
@@ -726,19 +732,22 @@ class TestJp2k_2_1(unittest.TestCase):
         with open(self.jp2file, 'rb') as fptr:
             data = fptr.read()
             with tempfile.NamedTemporaryFile(suffix='.jp2') as tfile:
-                # Codestream starts at byte 3127. SIZ marker at 3137.
-                # COD marker at 3186.  Subsampling at 3180.
-                tfile.write(data[0:3179])
+                # Codestream starts at byte 3323. SIZ marker at 3233.
+                # COD marker at 3282.  Subsampling at 3276.
+                offset = 3223
+                tfile.write(data[0:offset+52])
 
                 # Make the DY bytes of the SIZ segment zero.  That means that
                 # a subsampling factor is zero, which is illegal.
                 tfile.write(b'\x00')
-                tfile.write(data[3180:3182])
+                tfile.write(data[offset+53:offset+55])
                 tfile.write(b'\x00')
-                tfile.write(data[3184:3186])
+                tfile.write(data[offset+57:offset+59])
+                #tfile.write(data[3184:3186])
                 tfile.write(b'\x00')
 
-                tfile.write(data[3186:])
+                tfile.write(data[offset+59:])
+                #tfile.write(data[3186:])
                 tfile.flush()
                 with warnings.catch_warnings():
                     warnings.simplefilter("ignore")
@@ -753,6 +762,169 @@ class TestJp2k_2_1(unittest.TestCase):
                     with self.assertRaisesRegex((IOError, OSError), regexp):
                         j.read(rlevel=1)
 
+@unittest.skipIf(OPJ_DATA_ROOT is None,
+                 "OPJ_DATA_ROOT environment variable not set")
+class TestParsing(unittest.TestCase):
+    """Tests for verifying how parsing may be altered."""
+    def setUp(self):
+        self.jp2file = glymur.data.nemo()
+        # Reset parseoptions for every test.
+        glymur.set_parseoptions(codestream=True)
+
+    def tearDown(self):
+        pass
+
+    @unittest.skipIf(sys.platform.startswith('linux'), 'Failing on linux')
+    def test_bad_rsiz(self):
+        """Should not warn if RSIZ when parsing is turned off."""
+        # Actually there are three warning triggered by this codestream.
+        filename = opj_data_file('input/nonregression/edf_c2_1002767.jp2')
+        glymur.set_parseoptions(codestream=False)
+        with warnings.catch_warnings(record=True) as w:
+            j = Jp2k(filename)
+            self.assertEqual(len(w), 0)
+
+        glymur.set_parseoptions(codestream=True)
+        with warnings.catch_warnings(record=True) as w:
+            jp2 = Jp2k(filename)
+            self.assertTrue(issubclass(w[0].category, UserWarning))
+            self.assertTrue('Invalid profile' in str(w[0].message))
+
+    def test_main_header(self):
+        """Verify that the main header is not loaded when parsing turned off."""
+        # The hidden _main_header attribute should show up after accessing it.
+        glymur.set_parseoptions(codestream=False)
+        jp2 = Jp2k(self.jp2file)
+        jp2c = jp2.box[4]
+        self.assertIsNone(jp2c._main_header)
+        main_header = jp2c.main_header
+        self.assertIsNotNone(jp2c._main_header)
+
+@unittest.skipIf(OPJ_DATA_ROOT is None,
+                 "OPJ_DATA_ROOT environment variable not set")
+class TestJp2kOpjDataRootWarnings(unittest.TestCase):
+    """These tests should be run by just about all configuration."""
+
+    def test_undecodeable_box_id(self):
+        """Should warn in case of undecodeable box ID but not error out."""
+        filename = opj_data_file('input/nonregression/edf_c2_1013627.jp2')
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+            jp2 = Jp2k(filename)
+            self.assertTrue(issubclass(w[0].category, UserWarning))
+            self.assertTrue('Unrecognized box' in str(w[0].message))
+
+        # Now make sure we got all of the boxes.  Ignore the last, which was
+        # bad.
+        box_ids = [box.box_id for box in jp2.box[:-1]]
+        self.assertEqual(box_ids, ['jP  ', 'ftyp', 'jp2h', 'jp2c'])
+
+    def test_bad_ftyp_brand(self):
+        """Should warn in case of bad ftyp brand."""
+        filename = opj_data_file('input/nonregression/edf_c2_1000290.jp2')
+        with warnings.catch_warnings(record=True) as w:
+           warnings.simplefilter('always')
+           jp2 = Jp2k(filename)
+           self.assertTrue(issubclass(w[0].category, UserWarning))
+
+    def test_invalid_approximation(self):
+        """Should warn in case of invalid approximation."""
+        filename = opj_data_file('input/nonregression/edf_c2_1015644.jp2')
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+            jp2 = Jp2k(filename)
+            self.assertTrue(issubclass(w[0].category, UserWarning))
+            self.assertTrue('Invalid approximation' in str(w[0].message))
+
+    @unittest.skipIf(sys.platform.startswith('linux'), 'Failing on linux')
+    def test_invalid_colorspace(self):
+        """Should warn in case of invalid colorspace."""
+        filename = opj_data_file('input/nonregression/edf_c2_1103421.jp2')
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+            jp2 = Jp2k(filename)
+            self.assertTrue(issubclass(w[1].category, UserWarning))
+            self.assertTrue('Unrecognized colorspace' in str(w[1].message))
+
+    def test_stupid_windows_eol_at_end(self):
+        """Garbage characters at the end of the file."""
+        filename = opj_data_file('input/nonregression/issue211.jp2')
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+            jp2 = Jp2k(filename)
+            self.assertTrue(issubclass(w[1].category, UserWarning))
+
+
+@unittest.skipIf(OPJ_DATA_ROOT is None,
+                 "OPJ_DATA_ROOT environment variable not set")
+class TestJp2kOpjDataRoot(unittest.TestCase):
+    """These tests should be run by just about all configuration."""
+
+    @unittest.skipIf(os.name == "nt", "NamedTemporaryFile issue on windows")
+    def test_irreversible(self):
+        """Irreversible"""
+        filename = opj_data_file('input/nonregression/issue141.rawl')
+        expdata = np.fromfile(filename, dtype=np.uint16)
+        expdata.resize((2816, 2048))
+        with tempfile.NamedTemporaryFile(suffix='.j2k') as tfile:
+            j = Jp2k(tfile.name, 'wb')
+            j.write(expdata, irreversible=True)
+
+            codestream = j.get_codestream()
+            self.assertEqual(codestream.segment[2].spcod[8],
+                             glymur.core.WAVELET_XFORM_9X7_IRREVERSIBLE)
+
+            actdata = j.read()
+            self.assertTrue(fixtures.mse(actdata, expdata) < 250)
+
+    def test_no_cxform_pclr_jp2(self):
+        """Indices for pclr jpxfile if no color transform"""
+        filename = opj_data_file('input/conformance/file9.jp2')
+        j = Jp2k(filename)
+        rgb = j.read()
+        idx = j.read(ignore_pclr_cmap_cdef=True)
+        self.assertEqual(rgb.shape, (512, 768, 3))
+        self.assertEqual(idx.shape, (512, 768))
+
+        # Should be able to manually reconstruct the RGB image from the palette
+        # and indices.
+        palette = j.box[2].box[1].palette
+        rgb_from_idx = np.zeros(rgb.shape, dtype=np.uint8)
+        for r in np.arange(rgb.shape[0]):
+            for c in np.arange(rgb.shape[1]):
+                rgb_from_idx[r, c] = palette[idx[r, c]]
+        np.testing.assert_array_equal(rgb, rgb_from_idx)
+
+    def test_read_differing_subsamples(self):
+        """should error out with read used on differently subsampled images"""
+        # Verify that we error out appropriately if we use the read method
+        # on an image with differing subsamples
+        #
+        # Issue 86.
+        filename = opj_data_file('input/conformance/p0_05.j2k')
+        j = Jp2k(filename)
+        with self.assertRaises(RuntimeError):
+            j.read()
+
+    def test_no_cxform_cmap(self):
+        """Bands as physically ordered, not as physically intended"""
+        # This file has the components physically reversed.  The cmap box
+        # tells the decoder how to order them, but this flag prevents that.
+        filename = opj_data_file('input/conformance/file2.jp2')
+        with warnings.catch_warnings():
+            # The file has a bad compatibility list entry.  Not important here.
+            warnings.simplefilter("ignore")
+            j = Jp2k(filename)
+        ycbcr = j.read()
+        crcby = j.read(ignore_pclr_cmap_cdef=True)
+
+        expected = np.zeros(ycbcr.shape, ycbcr.dtype)
+        for k in range(crcby.shape[2]):
+            expected[:,:,crcby.shape[2] - k - 1] = crcby[:,:,k]
+
+        np.testing.assert_array_equal(ycbcr, expected)
+
+
 
 if __name__ == "__main__":
     unittest.main()
