@@ -27,16 +27,9 @@ suite.
 # asserWarns introduced in python 3.2 (python2.7/pylint issue)
 # pylint: disable=E1101
 
-# unittest2 is python2.6 only (pylint/python-2.7)
-# pylint: disable=F0401
-
 import re
 import sys
-
-if sys.hexversion < 0x02070000:
-    import unittest2 as unittest
-else:
-    import unittest
+import unittest
 
 import warnings
 
@@ -45,7 +38,7 @@ import numpy as np
 from glymur import Jp2k
 import glymur
 
-from .fixtures import OPENJP2_IS_V2_OFFICIAL, OPJ_DATA_ROOT
+from .fixtures import OPJ_DATA_ROOT
 from .fixtures import mse, peak_tolerance, read_pgx, opj_data_file
 
 
@@ -62,87 +55,6 @@ class TestSuite(unittest.TestCase):
     def tearDown(self):
         pass
 
-    def test_ETS_C0P0_p0_01_j2k(self):
-        jfile = opj_data_file('input/conformance/p0_01.j2k')
-        jp2k = Jp2k(jfile)
-        jpdata = jp2k.read(rlevel=0)
-
-        pgxfile = opj_data_file('baseline/conformance/c0p0_01.pgx')
-        pgxdata = read_pgx(pgxfile)
-        np.testing.assert_array_equal(jpdata, pgxdata)
-
-    def test_ETS_C0P0_p0_02_j2k(self):
-        jfile = opj_data_file('input/conformance/p0_02.j2k')
-        jp2k = Jp2k(jfile)
-        with warnings.catch_warnings():
-            # Invalid marker ID.
-            warnings.simplefilter("ignore")
-            jpdata = jp2k.read(rlevel=0)
-
-        pgxfile = opj_data_file('baseline/conformance/c0p0_02.pgx')
-        pgxdata = read_pgx(pgxfile)
-        np.testing.assert_array_equal(jpdata, pgxdata)
-
-    def test_ETS_C0P0_p0_03_j2k_r1(self):
-        jfile = opj_data_file('input/conformance/p0_03.j2k')
-        jp2k = Jp2k(jfile)
-        jpdata = jp2k.read(rlevel=1)
-
-        pgxfile = opj_data_file('baseline/conformance/c0p0_03r1.pgx')
-        pgxdata = read_pgx(pgxfile)
-        np.testing.assert_array_equal(jpdata, pgxdata)
-
-    def test_ETS_C0P0_p0_09_j2k(self):
-        jfile = opj_data_file('input/conformance/p0_09.j2k')
-        jp2k = Jp2k(jfile)
-        jpdata = jp2k.read(rlevel=2)
-
-        pgxfile = opj_data_file('baseline/conformance/c0p0_09.pgx')
-        pgxdata = read_pgx(pgxfile)
-
-        self.assertTrue(peak_tolerance(jpdata, pgxdata) < 4)
-        self.assertTrue(mse(jpdata, pgxdata) < 1.47)
-
-    def test_ETS_C0P0_p0_11_j2k(self):
-        jfile = opj_data_file('input/conformance/p0_11.j2k')
-        jp2k = Jp2k(jfile)
-        jpdata = jp2k.read(rlevel=0)
-
-        pgxfile = opj_data_file('baseline/conformance/c0p0_11.pgx')
-        pgxdata = read_pgx(pgxfile)
-
-        np.testing.assert_array_equal(jpdata, pgxdata)
-
-    def test_ETS_C0P0_p0_15_j2k_r1(self):
-        jfile = opj_data_file('input/conformance/p0_15.j2k')
-        jp2k = Jp2k(jfile)
-        jpdata = jp2k.read(rlevel=1)
-
-        pgxfile = opj_data_file('baseline/conformance/c0p0_15r1.pgx')
-        pgxdata = read_pgx(pgxfile)
-
-        np.testing.assert_array_equal(jpdata, pgxdata)
-
-    def test_ETS_C0P0_p0_16_j2k(self):
-        jfile = opj_data_file('input/conformance/p0_16.j2k')
-        jp2k = Jp2k(jfile)
-        jpdata = jp2k.read(rlevel=0)
-
-        pgxfile = opj_data_file('baseline/conformance/c0p0_16.pgx')
-        pgxdata = read_pgx(pgxfile)
-
-        np.testing.assert_array_equal(jpdata, pgxdata)
-
-    def test_ETS_C0P1_p1_01_j2k(self):
-        jfile = opj_data_file('input/conformance/p1_01.j2k')
-        jp2k = Jp2k(jfile)
-        jpdata = jp2k.read(rlevel=0)
-
-        pgxfile = opj_data_file('baseline/conformance/c0p1_01.pgx')
-        pgxdata = read_pgx(pgxfile)
-
-        np.testing.assert_array_equal(jpdata, pgxdata)
-
     def test_ETS_C1P0_p0_01_j2k(self):
         jfile = opj_data_file('input/conformance/p0_01.j2k')
         jp2k = Jp2k(jfile)
@@ -153,16 +65,6 @@ class TestSuite(unittest.TestCase):
 
         np.testing.assert_array_equal(jpdata, pgxdata)
 
-    def test_ETS_C1P0_p0_02_j2k(self):
-        jfile = opj_data_file('input/conformance/p0_02.j2k')
-        jp2k = Jp2k(jfile)
-        jpdata = jp2k.read(rlevel=0)
-
-        pgxfile = opj_data_file('baseline/conformance/c1p0_02_0.pgx')
-        pgxdata = read_pgx(pgxfile)
-
-        np.testing.assert_array_equal(jpdata, pgxdata)
-
     def test_ETS_C1P0_p0_03_j2k(self):
         jfile = opj_data_file('input/conformance/p0_03.j2k')
         jp2k = Jp2k(jfile)
@@ -193,6 +95,24 @@ class TestSuite(unittest.TestCase):
         self.assertTrue(peak_tolerance(jpdata[:, :, 2], pgxdata) < 6)
         self.assertTrue(mse(jpdata[:, :, 2], pgxdata) < 1.07)
 
+    @unittest.skip("Known failure in OPENJPEG test suite operation.")
+    def test_ETS_C1P0_p0_07_j2k(self):
+        jfile = opj_data_file('input/conformance/p0_07.j2k')
+        jp2k = Jp2k(jfile)
+        jpdata = jp2k.read()
+
+        pgxfile = opj_data_file('baseline/conformance/c1p0_07_0.pgx')
+        pgxdata = read_pgx(pgxfile)
+        np.testing.assert_array_equal(jpdata[:, :, 0], pgxdata)
+
+        pgxfile = opj_data_file('baseline/conformance/c1p0_07_1.pgx')
+        pgxdata = read_pgx(pgxfile)
+        np.testing.assert_array_equal(jpdata[:, : 1], pgxdata)
+
+        pgxfile = opj_data_file('baseline/conformance/c1p0_07_2.pgx')
+        pgxdata = read_pgx(pgxfile)
+        np.testing.assert_array_equal(jpdata[:, : 2], pgxdata)
+
     def test_ETS_C1P0_p0_08_j2k(self):
         jfile = opj_data_file('input/conformance/p0_08.j2k')
         jp2k = Jp2k(jfile)
@@ -304,7 +224,10 @@ class TestSuite(unittest.TestCase):
 
     def test_ETS_JP2_file1(self):
         jfile = opj_data_file('input/conformance/file1.jp2')
-        jp2k = Jp2k(jfile)
+        with warnings.catch_warnings():
+            # Bad compatibility list item.
+            warnings.simplefilter("ignore")
+            jp2k = Jp2k(jfile)
         jpdata = jp2k.read()
         self.assertEqual(jpdata.shape, (512, 768, 3))
 
@@ -332,7 +255,11 @@ class TestSuite(unittest.TestCase):
 
     def test_ETS_JP2_file5(self):
         jfile = opj_data_file('input/conformance/file5.jp2')
-        jp2k = Jp2k(jfile)
+        with warnings.catch_warnings():
+            # There's a warning for an unknown compatibility entry.
+            # Ignore it here.
+            warnings.simplefilter("ignore")
+            jp2k = Jp2k(jfile)
         jpdata = jp2k.read()
         self.assertEqual(jpdata.shape, (512, 768, 3))
 
@@ -358,11 +285,7 @@ class TestSuite(unittest.TestCase):
         jfile = opj_data_file('input/conformance/file9.jp2')
         jp2k = Jp2k(jfile)
         jpdata = jp2k.read()
-        if re.match(r"""1\.3""", glymur.version.openjpeg_version):
-            # Version 1.3 reads the indexed image as indices, not as RGB.
-            self.assertEqual(jpdata.shape, (512, 768))
-        else:
-            self.assertEqual(jpdata.shape, (512, 768, 3))
+        self.assertEqual(jpdata.shape, (512, 768, 3))
 
     def test_NR_DEC_Bretagne2_j2k_1_decode(self):
         jfile = opj_data_file('input/nonregression/Bretagne2.j2k')
@@ -376,34 +299,6 @@ class TestSuite(unittest.TestCase):
         jp2.read()
         self.assertTrue(True)
 
-    @unittest.skipIf(re.match(r"""1\.5\.2""", glymur.version.openjpeg_version),
-                     "Test fails in 1.5.2")
-    @unittest.skipIf(sys.hexversion < 0x03020000,
-                     "Uses features introduced in 3.2.")
-    def test_NR_DEC_broken_jp2_4_decode(self):
-        jfile = opj_data_file('input/nonregression/broken.jp2')
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
-            # colr box has bad length.
-            jp2 = Jp2k(jfile)
-            self.assertEqual(len(w), 2)
-        with self.assertRaises(IOError):
-            jp2.read()
-        self.assertTrue(True)
-
-    @unittest.skipIf(re.match(r"""1\.5\.2""", glymur.version.openjpeg_version),
-                     "Test fails in 1.5.2")
-    def test_NR_DEC_broken3_jp2_6_decode(self):
-        jfile = opj_data_file('input/nonregression/broken3.jp2')
-        with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
-            # colr box has bad length.
-            j = Jp2k(jfile)
-            self.assertEqual(len(w), 2)
-
-        with self.assertRaises(IOError):
-            j.read()
-
     def test_NR_DEC_buxI_j2k_9_decode(self):
         jfile = opj_data_file('input/nonregression/buxI.j2k')
         Jp2k(jfile).read()
@@ -517,27 +412,6 @@ class TestSuite_bands(unittest.TestCase):
     def tearDown(self):
         pass
 
-    def test_ETS_C0P0_p0_05_j2k(self):
-        jfile = opj_data_file('input/conformance/p0_05.j2k')
-        jp2k = Jp2k(jfile)
-        jpdata = jp2k.read_bands(rlevel=3)
-
-        pgxfile = opj_data_file('baseline/conformance/c0p0_05.pgx')
-        pgxdata = read_pgx(pgxfile)
-        self.assertTrue(peak_tolerance(jpdata[0], pgxdata) < 54)
-        self.assertTrue(mse(jpdata[0], pgxdata) < 68)
-
-    def test_ETS_C0P1_p1_03_j2k(self):
-        jfile = opj_data_file('input/conformance/p1_03.j2k')
-        jp2k = Jp2k(jfile)
-        jpdata = jp2k.read_bands(rlevel=3)
-
-        pgxfile = opj_data_file('baseline/conformance/c0p1_03.pgx')
-        pgxdata = read_pgx(pgxfile)
-
-        self.assertTrue(peak_tolerance(jpdata[0], pgxdata) < 28)
-        self.assertTrue(mse(jpdata[0], pgxdata) < 18.8)
-
     def test_ETS_C1P1_p1_03_j2k(self):
         jfile = opj_data_file('input/conformance/p1_03.j2k')
         jp2k = Jp2k(jfile)
