@@ -2,25 +2,28 @@
 The tests here do not correspond directly to the OpenJPEG test suite, but
 seem like logical negative tests to add.
 """
+# E1101:  assertWarns introduced in python 3.2
+# pylint: disable=E1101
+
 # R0904:  Not too many methods in unittest.
 # pylint: disable=R0904
 
-# unittest2 is python2.6 only (pylint/python-2.7)
-# pylint: disable=F0401
-
 import os
 import re
 import sys
 import tempfile
+import unittest
 import warnings
 
-if sys.hexversion < 0x02070000:
-    import unittest2 as unittest
-else:
-    import unittest
-
 import numpy as np
 
+try:
+    import skimage.io
+    skimage.io.use_plugin('freeimage', 'imread')
+    _HAS_SKIMAGE_FREEIMAGE_SUPPORT = True
+except ((ImportError, RuntimeError)):
+    _HAS_SKIMAGE_FREEIMAGE_SUPPORT = False
+
 from .fixtures import OPJ_DATA_ROOT, opj_data_file, read_image
 from .fixtures import NO_READ_BACKEND, NO_READ_BACKEND_MSG
 
@@ -28,8 +31,6 @@ from glymur import Jp2k
 import glymur
 
 
-@unittest.skipIf(re.match(r"""1\.[01234]""", glymur.version.openjpeg_version),
-                 "Functionality not implemented for 1.3, 1.4")
 @unittest.skipIf(OPJ_DATA_ROOT is None,
                  "OPJ_OPJ_DATA_ROOT environment variable not set")
 class TestSuiteNegative(unittest.TestCase):
@@ -42,6 +43,21 @@ class TestSuiteNegative(unittest.TestCase):
     def tearDown(self):
         pass
 
+
+    @unittest.skipIf(not _HAS_SKIMAGE_FREEIMAGE_SUPPORT,
+                     "Cannot read input image without scikit-image/freeimage")
+    @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
+    def test_cinema2K_bad_frame_rate(self):
+        """Cinema2k frame rate must be either 24 or 48."""
+        relfile = 'input/nonregression/X_5_2K_24_235_CBR_STEM24_000.tif'
+        infile = opj_data_file(relfile)
+        data = skimage.io.imread(infile)
+        with tempfile.NamedTemporaryFile(suffix='.j2k') as tfile:
+            j = Jp2k(tfile.name, 'wb')
+            with self.assertRaises(IOError):
+                j.write(data, cinema2k=36)
+
+
     @unittest.skipIf(NO_READ_BACKEND, NO_READ_BACKEND_MSG)
     @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
     def test_psnr_with_cratios(self):
@@ -68,9 +84,8 @@ class TestSuiteNegative(unittest.TestCase):
         jfile = opj_data_file(relpath)
         jp2k = Jp2k(jfile)
         with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("always")
+            warnings.simplefilter('ignore')
             codestream = jp2k.get_codestream(header_only=False)
-            self.assertEqual(len(w), 1)
 
         # Verify that the last segment returned in the codestream is SOD,
         # not EOC.  Codestream parsing should stop when we try to jump to
@@ -112,7 +127,7 @@ class TestSuiteNegative(unittest.TestCase):
         infile = os.path.join(OPJ_DATA_ROOT,
                               'input/nonregression/mem-b2ace68c-1381.jp2')
         with warnings.catch_warnings(record=True) as w:
-            warnings.simplefilter("ignore")
+            warnings.simplefilter('ignore')
             Jp2k(infile)
 
     @unittest.skipIf(os.name == "nt", "Temporary file issue on window.")
