@@ -1,21 +1,25 @@
-# This file is part of glymur, a Python interface for accessing JPEG 2000.
-#
-# http://glymur.readthedocs.org
-#
-# Copyright 2013 John Evans
-#
-# License:  MIT
+"""
+This file is part of glymur, a Python interface for accessing JPEG 2000.
+
+http://glymur.readthedocs.org
+
+Copyright 2013 John Evans
+
+License:  MIT
+"""
 
 import sys
-import numpy as np
 from distutils.version import LooseVersion
 
+import lxml.etree
+import numpy as np
+
 from .lib import openjpeg as opj
 from .lib import openjp2 as opj2
 
 # Do not change the format of this next line!  Doing so risks breaking
 # setup.py
-version = "0.5.12"
+version = "0.6.1"
 _sv = LooseVersion(version)
 version_tuple = _sv.version
 
@@ -46,10 +50,12 @@ OPENJPEG      {openjpeg}
 Python        {python}
 sys.platform  {platform}
 sys.maxsize   {maxsize}
+lxml          {elxml}
 numpy         {numpy}
 """.format(glymur=version,
            openjpeg=openjpeg_version,
            python=sys.version,
            platform=sys.platform,
            maxsize=sys.maxsize,
+           elxml=lxml.etree.__version__,
            numpy=np.__version__)
